@@ -50,6 +50,7 @@ import Language.Haskell.Exts.Comments
 import Data.List
 import Data.Maybe (fromMaybe)
 import Language.Preprocessor.Unlit
+import System.IO
 
 -- | Parse a source file on disk, using the default parse mode.
 parseFile :: FilePath -> IO (ParseResult Module)
@@ -62,17 +63,17 @@ parseFileWithExts exts fp = parseFileWithMode (defaultParseMode { extensions = e
 
 -- | Parse a source file on disk, supplying a custom parse mode.
 parseFileWithMode :: ParseMode -> FilePath -> IO (ParseResult Module)
-parseFileWithMode p fp = readFile fp >>= return . parseFileContentsWithMode p
+parseFileWithMode p fp = readUTF8File fp >>= return . parseFileContentsWithMode p
 
 -- | Parse a source file on disk, supplying a custom parse mode, and retaining comments.
 parseFileWithComments :: ParseMode -> FilePath -> IO (ParseResult (Module, [Comment]))
-parseFileWithComments p fp = readFile fp >>= return . parseFileContentsWithComments p
+parseFileWithComments p fp = readUTF8File fp >>= return . parseFileContentsWithComments p
 
 -- | Parse a source file on disk, supplying a custom parse mode, and retaining comments
 --  as well as unknown pragmas.
 parseFileWithCommentsAndPragmas :: ParseMode -> FilePath -> IO (ParseResult (Module, [Comment], [UnknownPragma]))
 parseFileWithCommentsAndPragmas p fp =
-    readFile fp >>= return . parseFileContentsWithCommentsAndPragmas p
+    readUTF8File fp >>= return . parseFileContentsWithCommentsAndPragmas p
 
 -- | Parse a source file from a string using the default parse mode.
 parseFileContents :: String -> ParseResult Module
@@ -85,22 +86,22 @@ parseFileContentsWithExts exts = parseFileContentsWithMode (defaultParseMode { e
 
 -- | Parse a source file from a string using a custom parse mode.
 parseFileContentsWithMode :: ParseMode -> String -> ParseResult Module
-parseFileContentsWithMode p@(ParseMode fn oldLang exts ign _ _) rawStr =
+parseFileContentsWithMode p@(ParseMode fn oldLang exts ign _ _ _) rawStr =
         let md = delit fn $ ppContents rawStr
-            (bLang, extraExts) = 
+            (bLang, extraExts) =
                 case (ign, readExtensions md) of
-                  (False, Just (mLang, es)) -> 
+                  (False, Just (mLang, es)) ->
                        (fromMaybe oldLang mLang, es)
                   _ -> (oldLang, [])
          in parseWithMode (p { baseLanguage = bLang, extensions = exts ++ extraExts }) md
 
 -- | Parse a source file from a string using a custom parse mode and retaining comments.
 parseFileContentsWithComments :: ParseMode -> String -> ParseResult (Module, [Comment])
-parseFileContentsWithComments p@(ParseMode fn oldLang exts ign _ _) rawStr =
+parseFileContentsWithComments p@(ParseMode fn oldLang exts ign _ _ _) rawStr =
         let md = delit fn $ ppContents rawStr
-            (bLang, extraExts) = 
+            (bLang, extraExts) =
                 case (ign, readExtensions md) of
-                  (False, Just (mLang, es)) -> 
+                  (False, Just (mLang, es)) ->
                        (fromMaybe oldLang mLang, es)
                   _ -> (oldLang, [])
          in parseWithComments (p { baseLanguage = bLang, extensions = exts ++ extraExts }) md
@@ -135,7 +136,7 @@ readExtensions str = case getTopPragmas str of
         getExts (LanguagePragma _ ns) = map readExt ns
         getExts _ = []
 
-        readExt (Ident e) = 
+        readExt (Ident e) =
             case classifyLanguage e of
               UnknownLanguage _ -> Right $ classifyExtension e
               lang -> Left lang
@@ -171,3 +172,9 @@ separatePragmas r =
                       pragLike (Comment b _ s) = b && pcond s
                       pcond s = length s > 1 && take 1 s == "#" && last s == '#'
         ParseFailed l s ->  ParseFailed l s
+
+readUTF8File :: FilePath -> IO String
+readUTF8File fp = do
+  h <- openFile fp ReadMode
+  hSetEncoding h utf8
+  hGetContents h
