@@ -25,6 +25,8 @@ module Language.Haskell.Exts.Annotated (
     , module Language.Haskell.Exts.SrcLoc
     , module Language.Haskell.Exts.Comments
     , module Language.Haskell.Exts.Extension
+    , module Language.Haskell.Exts.Annotated.Parser
+    , module Language.Haskell.Exts.Annotated.Comments
     -- * Parsing of Haskell source files
     , parseFile
     , parseFileWithMode
@@ -34,18 +36,13 @@ module Language.Haskell.Exts.Annotated (
     , parseFileContentsWithMode
     , parseFileContentsWithExts
     , parseFileContentsWithComments
-    -- * Parsing of Haskell source elements,
-    , parseModule, parseModuleWithMode, parseModuleWithComments
-    , parseExp, parseExpWithMode, parseExpWithComments
-    , parseStmt, parseStmtWithMode, parseStmtWithComments
-    , parsePat, parsePatWithMode, parsePatWithComments
-    , parseDecl, parseDeclWithMode, parseDeclWithComments
-    , parseType, parseTypeWithMode, parseTypeWithComments
     -- * Read extensions declared in LANGUAGE pragmas
     , readExtensions
     ) where
 
 import Language.Haskell.Exts.Annotated.Build
+import Language.Haskell.Exts.Annotated.Comments
+import Language.Haskell.Exts.Annotated.Parser
 import Language.Haskell.Exts.Annotated.Syntax
 import Language.Haskell.Exts.Parser ( Parseable(..), ParseResult(..), fromParseResult, ParseMode(..), defaultParseMode )
 import Language.Haskell.Exts.Lexer ( lexTokenStream, lexTokenStreamWithMode, Token(..) )
@@ -56,11 +53,10 @@ import Language.Haskell.Exts.SrcLoc
 import Language.Haskell.Exts.Extension
 import Language.Haskell.Exts.Comments
 
-import Language.Haskell.Exts.InternalParser
-
 import Data.List
 import Data.Maybe (fromMaybe)
 import Language.Preprocessor.Unlit
+import System.IO
 
 -- | Parse a source file on disk, using the default parse mode.
 parseFile :: FilePath -> IO (ParseResult (Module SrcSpanInfo))
@@ -69,17 +65,17 @@ parseFile fp = parseFileWithMode (defaultParseMode { parseFilename = fp }) fp
 -- | Parse a source file on disk, with an extra set of extensions to know about
 --   on top of what the file itself declares.
 parseFileWithExts :: [Extension] -> FilePath -> IO (ParseResult (Module SrcSpanInfo))
-parseFileWithExts exts fp = 
-    parseFileWithMode (defaultParseMode { 
-                         extensions = exts, 
+parseFileWithExts exts fp =
+    parseFileWithMode (defaultParseMode {
+                         extensions = exts,
                          parseFilename = fp }) fp
 
 -- | Parse a source file on disk, supplying a custom parse mode.
 parseFileWithMode :: ParseMode -> FilePath -> IO (ParseResult (Module SrcSpanInfo))
-parseFileWithMode p fp = readFile fp >>= return . parseFileContentsWithMode p
+parseFileWithMode p fp = readUTF8File fp >>= return . parseFileContentsWithMode p
 
 parseFileWithComments :: ParseMode -> FilePath -> IO (ParseResult (Module SrcSpanInfo, [Comment]))
-parseFileWithComments p fp = readFile fp >>= return . parseFileContentsWithComments p
+parseFileWithComments p fp = readUTF8File fp >>= return . parseFileContentsWithComments p
 
 -- | Parse a source file from a string using the default parse mode.
 parseFileContents :: String -> ParseResult (Module SrcSpanInfo)
@@ -88,27 +84,27 @@ parseFileContents = parseFileContentsWithMode defaultParseMode
 -- | Parse a source file from a string, with an extra set of extensions to know about
 --   on top of what the file itself declares.
 parseFileContentsWithExts :: [Extension] -> String -> ParseResult (Module SrcSpanInfo)
-parseFileContentsWithExts exts = 
+parseFileContentsWithExts exts =
     parseFileContentsWithMode (defaultParseMode { extensions = exts })
 
 -- | Parse a source file from a string using a custom parse mode.
 parseFileContentsWithMode :: ParseMode -> String -> ParseResult (Module SrcSpanInfo)
-parseFileContentsWithMode p@(ParseMode fn oldLang exts ign _ _) rawStr =
+parseFileContentsWithMode p@(ParseMode fn oldLang exts ign _ _ _) rawStr =
         let md = delit fn $ ppContents rawStr
-            (bLang, extraExts) = 
+            (bLang, extraExts) =
                 case (ign, readExtensions md) of
-                  (False, Just (mLang, es)) -> 
+                  (False, Just (mLang, es)) ->
                        (fromMaybe oldLang mLang, es)
                   _ -> (oldLang, [])
          in -- trace (fn ++ ": " ++ show extraExts) $
               parseModuleWithMode (p { baseLanguage = bLang, extensions = exts ++ extraExts }) md
 
 parseFileContentsWithComments :: ParseMode -> String -> ParseResult (Module SrcSpanInfo, [Comment])
-parseFileContentsWithComments p@(ParseMode fn oldLang exts ign _ _) rawStr =
+parseFileContentsWithComments p@(ParseMode fn oldLang exts ign _ _ _) rawStr =
         let md = delit fn $ ppContents rawStr
-            (bLang, extraExts) = 
+            (bLang, extraExts) =
                 case (ign, readExtensions md) of
-                  (False, Just (mLang, es)) -> 
+                  (False, Just (mLang, es)) ->
                        (fromMaybe oldLang mLang, es)
                   _ -> (oldLang, [])
          in parseModuleWithComments (p { baseLanguage = bLang, extensions = exts ++ extraExts }) md
@@ -124,7 +120,7 @@ readExtensions str = case getTopPragmas str of
         getExts (LanguagePragma _ ns) = map readExt ns
         getExts _ = []
 
-        readExt (Ident _ e) = 
+        readExt (Ident _ e) =
             case classifyLanguage e of
               UnknownLanguage _ -> Right $ classifyExtension e
               lang -> Left lang
@@ -146,3 +142,9 @@ ppContents = unlines . f . lines
 
 delit :: String -> String -> String
 delit fn = if ".lhs" `isSuffixOf` fn then unlit fn else id
+
+readUTF8File :: FilePath -> IO String
+readUTF8File fp = do
+  h <- openFile fp ReadMode
+  hSetEncoding h utf8
+  hGetContents h
