@@ -113,6 +113,7 @@ infixApp = InfixApp
 appFun :: [l] -> Exp l -> [Exp l] -> Exp l
 appFun _ f [] = f
 appFun (l:ls) f (a:as) = appFun ls (app l f a) as
+appFun [] _ (_:_) = error "haskell-src-exts: Build: appFun: too few annotations supplied"
 
 -- | A constructor pattern, with argument patterns.
 pApp :: l -> Name l -> [Pat l] -> Pat l
@@ -152,15 +153,15 @@ intE l i = Lit l $ Int l i (show i)
 
 -- | A literal string pattern.
 strP :: l -> String -> Pat l
-strP l s = PLit l $ String l s s
+strP l s = PLit l (Signless l) $ String l s s
 
 -- | A literal character pattern.
 charP :: l -> Char -> Pat l
-charP l c = PLit l $ Char l c [c]
+charP l c = PLit l (Signless l) $ Char l c [c]
 
 -- | A literal integer pattern.
 intP :: l -> Integer -> Pat l
-intP l i = PLit l $ Int l i (show i)
+intP l i = PLit l (if i >= 0 then Signless l else Negative l) . Int l i . show . abs $ i
 
 -- | A do block formed by the given statements.
 --   The last statement in the list should be
@@ -190,15 +191,15 @@ altGW :: l -> Pat l -> [Stmt l] -> Exp l -> Binds l -> Alt l
 altGW l p gs e w = Alt l p (gAlt l gs e) (Just w)
 
 -- | An unguarded righthand side of a @case@ alternative.
-unGAlt :: l -> Exp l -> GuardedAlts l
-unGAlt = UnGuardedAlt
+unGAlt :: l -> Exp l -> Rhs l
+unGAlt = UnGuardedRhs
 
 -- | An list of guarded righthand sides for a @case@ alternative.
-gAlts :: l -> [([Stmt l], Exp l)] -> GuardedAlts l
-gAlts l as = GuardedAlts l $ map (\(gs,e) -> GuardedAlt l gs e) as
+gAlts :: l -> [([Stmt l], Exp l)] -> Rhs l
+gAlts l as = GuardedRhss l $ map (\(gs,e) -> GuardedRhs l gs e) as
 
 -- | A single guarded righthand side for a @case@ alternative.
-gAlt :: l -> [Stmt l] -> Exp l -> GuardedAlts l
+gAlt :: l -> [Stmt l] -> Exp l -> Rhs l
 gAlt l gs e = gAlts l [(gs,e)]
 
 -- | A list expression.
@@ -266,13 +267,13 @@ simpleFun l f a e = let rhs = UnGuardedRhs l e
 -- there are no guards and no 'where' clause.
 patBind :: l -> Pat l -> Exp l -> Decl l
 patBind l p e = let rhs = UnGuardedRhs l e
-         in PatBind l p Nothing rhs Nothing
+         in PatBind l p rhs Nothing
 
 -- | A pattern bind where the pattern is a variable, and where
 -- there are no guards, but with a 'where' clause.
 patBindWhere :: l -> Pat l -> Exp l -> [Decl l] -> Decl l
 patBindWhere l p e ds = let rhs = UnGuardedRhs l e
-             in PatBind l p Nothing rhs (Just $ binds l ds)
+             in PatBind l p rhs (Just $ binds l ds)
 
 -- | Bind an identifier to an expression.
 nameBind :: l -> Name l -> Exp l -> Decl l
@@ -280,7 +281,7 @@ nameBind l n e = patBind l (pvar l n) e
 
 -- | Apply function of a given name to a list of arguments.
 metaFunction :: l -> String -> [Exp l] -> Exp l
-metaFunction l s es = mf l s (reverse es)
+metaFunction l' s' es' = mf l' s' (reverse es')
   where mf l s []     = var l $ name l s
         mf l s (e:es) = app l (mf l s es) e
 
