@@ -43,7 +43,7 @@ newtype EP x = EP (Pos -> [Comment] -> (x, Pos, [Comment], ShowS))
 
 instance Functor EP where
   fmap = liftM
-  
+
 instance Applicative EP where
   pure = return
   (<*>) = ap
@@ -67,7 +67,12 @@ setPos :: Pos -> EP ()
 setPos l = EP (\_ cs -> ((),l,cs,id))
 
 printString :: String -> EP ()
-printString str = EP (\(l,c) cs -> ((), (l,c+length str), cs, showString str))
+printString str =
+  EP (\(l,c) cs -> let (l', c') = foldl go (l, c) str
+                       go (cl, _) '\n' = (cl + 1, 1)
+                       go (cl, cc) _    = (cl, cc + 1)
+                   in ((), (l', c'), cs, showString str))
+
 
 getComment :: EP (Maybe Comment)
 getComment = EP $ \l cs ->
@@ -353,15 +358,13 @@ instance ExactP CName where
 instance ExactP Namespace where
   exactP ns = case ns of
      NoNamespace _   -> return ()
-     TypeNamespace l ->
-        case srcInfoPoints l of
-            [a] -> printStringAt (pos a) "type"
-            _   -> errorEP "ExactP: Namespace is given too few srcInfoPoints"
+     TypeNamespace l -> printStringAt (pos l) "type"
+     PatternNamespace l -> printStringAt (pos l) "pattern"
 
 instance ExactP ExportSpec where
   exactP espec = case espec of
-     EVar _ t qn    -> exactPC t >> exactPC qn
-     EAbs _ qn      -> exactP qn
+     EVar _ qn    -> exactPC qn
+     EAbs _ ns qn -> exactP ns >> exactPC qn
      EThingAll l qn -> exactP qn >> printPoints l ["(","..",")"]
      EThingWith l qn cns    ->
         let k = length (srcInfoPoints l)
@@ -386,8 +389,8 @@ instance ExactP ImportSpecList where
 
 instance ExactP ImportSpec where
   exactP ispec = case ispec of
-    IVar _ t qn -> exactPC t >> exactPC qn
-    IAbs _ n    -> exactP n
+    IVar _ qn       -> exactPC qn
+    IAbs _ ns n     -> exactP ns >> exactPC n
     IThingAll l n   -> exactP n >> printPoints l ["(","..",")"]
     IThingWith l n cns    ->
         let k = length (srcInfoPoints l)
@@ -509,7 +512,11 @@ instance ExactP ModulePragma where
          in printInterleaved (zip pts ("{-# LANGUAGE":replicate k "," ++ replicate m "" ++ ["#-}"])) ns
     OptionsPragma    l mt str   ->
         let k = length (srcInfoPoints l)
-            opstr = "{-# OPTIONS" ++ case mt of { Just t -> "_" ++ show t ; _ -> "" } ++ str
+            -- We strip out a leading space in the lexer unless the pragma
+            -- starts with a newline.
+            addSpace xs@('\n':_) = xs
+            addSpace xs = ' ':xs
+            opstr = "{-# OPTIONS" ++ case mt of { Just t -> "_" ++ show t ; _ -> "" } ++ addSpace str
          in printPoints l $ opstr : replicate (k-2) "" ++ ["#-}"]
     AnnModulePragma  l ann'      ->
         case srcInfoPoints l of
@@ -704,12 +711,51 @@ instance ExactP Decl where
         let pts = srcInfoPoints l
         printInterleaved' (zip pts (replicate (length pts - 1) "," ++ ["::"])) ns
         exactPC t
+    PatSynSig l n dh c1 c2 t -> do
+      case srcInfoPoints l of
+        (pat:dc:pts1) -> do
+          printStringAt (pos pat) "pattern"
+          exactPC n
+          printStringAt (pos dc) "::"
+          case dh of
+            Nothing -> return ()
+            Just tvs ->
+              case pts1 of
+                (a:b:_) -> do
+                      printStringAt (pos a) "forall"
+                      mapM_ exactPC tvs
+                      printStringAt (pos b) "."
+                _ -> errorEP "ExactP: Decl: PatSynSig: Forall: is given too few srcInfoPoints"
+          maybeEP exactPC c1
+          maybeEP exactPC c2
+          exactPC t
+        _ -> errorEP "ExactP: Decl: PatSynSig: is given too few srcInfoPoints"
     FunBind      _ ms   -> mapM_ exactPC ms
     PatBind      l p rhs mbs -> do
         let pts = srcInfoPoints l
         exactP p
         exactPC rhs
         maybeEP (\bs -> printStringAt (pos (head pts)) "where" >> exactPC bs) mbs
+    PatSyn l lhs rhs dir ->
+      case srcInfoPoints l of
+        [pat,sepPos] -> do
+          let sep = case dir of
+                      ImplicitBidirectional     -> "="
+                      ExplicitBidirectional _ _ -> "<-"
+                      Unidirectional            -> "<-"
+          printStringAt (pos pat) "pattern"
+          exactPC lhs
+          printStringAt (pos sepPos) sep
+          exactPC rhs
+          case dir of
+            ExplicitBidirectional bl ds -> do
+              case srcInfoPoints bl of
+                (w:pts) -> do
+                  printStringAt (pos w) "where"
+                  layoutList pts ds
+                _ -> errorEP "ExactP: Decl: PaySyn: ExplicitBidirectional is given too few srcInfoPoints"
+            _ -> return ()
+        _ -> errorEP "ExactP: Decl: PatSyn is given too few srcInfoPoints"
     ForImp       l cc msf mstr n t   ->
         case srcInfoPoints l of
          _:b:pts -> do
@@ -826,6 +872,22 @@ instance ExactP Decl where
             maybeEP exactPC b
             printStringAt (pos b') "#-}"
          _ -> errorEP "ExactP: Decl: MinimalPragma is given wrong number of srcInfoPoints"
+    RoleAnnotDecl l ty roles ->
+      case srcInfoPoints l of
+         (t:r:_) -> do
+            printStringAt (pos t)  "type"
+            printStringAt (pos r)  "role"
+            exactPC ty
+            mapM_ exactPC roles
+         _ -> errorEP "ExactP: Decl: RoleAnnotDecl is given wrong number of srcInfoPoints"
+
+instance ExactP Role where
+  exactP r =
+    case r of
+      RoleWildcard l     -> printStringAt (pos l) "_"
+      Representational l -> printStringAt (pos l) "representational"
+      Phantom l          -> printStringAt (pos l) "phantom"
+      Nominal l          -> printStringAt (pos l) "nominal"
 
 
 instance ExactP Annotation where
@@ -943,7 +1005,6 @@ instance ExactP TyVarBind where
 instance ExactP Kind where
   exactP kd' = case kd' of
     KindStar  _     -> printString "*"
-    KindBang  _     -> printString "!"
     KindFn    l k1 k2 ->
         case srcInfoPoints l of
          [a] -> do
@@ -967,11 +1028,13 @@ instance ExactP Kind where
             e = ")"
             pts = srcInfoPoints l
         in printInterleaved (zip pts (o: replicate (length pts - 2) "," ++ [e])) ks
-    KindList  l ks ->
-        let o = "["
-            e = "]"
-            pts = srcInfoPoints l
-        in printInterleaved (zip pts (o: replicate (length pts - 2) "," ++ [e])) ks
+    KindList  l k ->
+      case srcInfoPoints l of
+        [_, close] -> do
+          printString "["
+          exactPC k
+          printStringAt (pos close) "]"
+        _ -> errorEP "ExactP: Kind: KindList is given wrong number of srcInfoPoints"
 
 
 
@@ -1043,6 +1106,7 @@ instance ExactP Type where
 
     TySplice _ sp  -> exactP sp
     TyBang _ b t -> exactPC b >> exactPC t
+    TyWildCard _ mn      -> printString "_" >> maybeEP exactPC mn
 
 instance ExactP Promoted where
   exactP (PromotedInteger _ _ rw) = printString rw
@@ -1091,7 +1155,7 @@ printContext ctxt = do
 instance ExactP Asst where
   exactP asst = case asst of
     ClassA _ qn ts -> exactP qn >> mapM_ exactPC ts
-    VarA _ n          -> exactPC n
+    AppA _ n ns       -> exactPC n >> mapM_ exactPC ns
     InfixA _ ta qn tb -> exactP ta >> epInfixQName qn >> exactPC tb
     IParam l ipn t    ->
         case srcInfoPoints l of
@@ -1114,6 +1178,7 @@ instance ExactP Asst where
             exactPC asst'
             printStringAt (pos b) ")"
          _ -> errorEP "ExactP: Asst: ParenA is given wrong number of srcInfoPoints"
+    WildCardA _ mn -> printString "_" >> maybeEP exactPC mn
 
 instance ExactP Deriving where
   exactP (Deriving l ihs) =
@@ -1635,6 +1700,7 @@ instance ExactP Exp where
             printStringAt (pos b) "case"
             layoutList pts alts
          _ -> errorEP "ExactP: Exp: LCase is given wrong number of srcInfoPoints"
+    ExprHole _ -> printString "_"
 
 instance ExactP FieldUpdate where
   exactP fup = case fup of
@@ -1975,13 +2041,14 @@ instance ExactP Binds where
   exactP (IPBinds l ips) = layoutList (srcInfoPoints l) ips
 
 instance ExactP CallConv where
-  exactP (StdCall   _) = printString "stdcall"
-  exactP (CCall     _) = printString "ccall"
-  exactP (CPlusPlus _) = printString "cplusplus"
-  exactP (DotNet    _) = printString "dotnet"
-  exactP (Jvm       _) = printString "jvm"
-  exactP (Js        _) = printString "js"
-  exactP (CApi      _) = printString "capi"
+  exactP (StdCall    _) = printString "stdcall"
+  exactP (CCall      _) = printString "ccall"
+  exactP (CPlusPlus  _) = printString "cplusplus"
+  exactP (DotNet     _) = printString "dotnet"
+  exactP (Jvm        _) = printString "jvm"
+  exactP (Js         _) = printString "js"
+  exactP (JavaScript _) = printString "javascript"
+  exactP (CApi       _) = printString "capi"
 
 instance ExactP Safety where
   exactP (PlayRisky _) = printString "unsafe"
