@@ -42,6 +42,7 @@ import Language.Haskell.Exts.Annotated.Simplify ( sQOp, sAssoc, sQName, sModuleH
 
 import Control.Monad (when, (<=<), liftM, liftM2, liftM3, liftM4)
 import Data.Traversable (mapM)
+import Data.Maybe (fromMaybe)
 import Prelude hiding (mapM)
 
 -- | All AST elements that may include expressions which in turn may
@@ -63,7 +64,7 @@ instance AppFixity Exp where
               let fixup (a1,p1) (a2,p2) y pre = do
                       when (p1 == p2 && (a1 /= a2 || a1 == S.AssocNone)) -- Ambiguous infix expression!
                            $ fail "Ambiguous infix expression"
-                      if (p1 > p2 || p1 == p2 && (a1 == S.AssocLeft || a2 == S.AssocNone)) -- Already right order
+                      if p1 > p2 || p1 == p2 && (a1 == S.AssocLeft || a2 == S.AssocNone) -- Already right order
                        then return $ InfixApp l2 e op2 z
                        else liftM pre (infFix fixs $ InfixApp (ann y <++> ann z) y op2 z)
               case e of
@@ -83,7 +84,7 @@ instance AppFixity Pat where
               let fixup (a1,p1) (a2,p2) y pre = do
                       when (p1 == p2 && (a1 /= a2 || a1 == S.AssocNone )) -- Ambiguous infix expression!
                            $ fail "Ambiguous infix expression"
-                      if (p1 > p2 || p1 == p2 && (a1 == S.AssocLeft || a2 == S.AssocNone)) -- Already right order
+                      if p1 > p2 || p1 == p2 && (a1 == S.AssocLeft || a2 == S.AssocNone) -- Already right order
                        then return $ PInfixApp l2 p op2 z
                        else liftM pre (infFix fixs $ PInfixApp (ann y <++> ann z) y op2 z)
               case p of
@@ -108,13 +109,11 @@ askFixityP xs qn = askFix xs (g $ sQName qn)
     where
         g (S.Special S.Cons) = S.UnQual (S.Symbol ":")
         g x                  = x
-        
+
 askFix :: [Fixity] -> S.QName -> (S.Assoc, Int)
 askFix xs = \k -> lookupWithDefault (S.AssocLeft, 9) k mp
     where
-        lookupWithDefault def k mp' = case lookup k mp' of
-            Nothing -> def
-            Just x  -> x
+        lookupWithDefault def k mp' = fromMaybe def $ lookup k mp'
 
         mp = [(x,(a,p)) | Fixity a p x <- xs]
 
@@ -142,7 +141,9 @@ instance AppFixity Decl where
         InstDecl  l olp ih idecls         -> liftM (InstDecl  l olp ih)  $ mapM (mapM fix) idecls
         SpliceDecl l spl        -> liftM (SpliceDecl l) $ fix spl
         FunBind l matches       -> liftM (FunBind l) $ mapM fix matches
-        PatBind l p rhs bs      -> liftM3 (PatBind l) (fix p) (fix rhs) (mapM fix bs)
+        PatBind l p rhs bs      ->
+         let extraFix x = applyFixities (fixs ++ maybe [] getBindFixities bs) x
+          in liftM3 (PatBind l) (extraFix p) (extraFix rhs) (mapM extraFix bs)
         AnnPragma l ann'         -> liftM (AnnPragma l) $ fix ann'
         _                       -> return decl
       where fix x = applyFixities fixs x
@@ -156,13 +157,24 @@ getFixities :: Maybe S.ModuleName -> [Decl l] -> [Fixity]
 getFixities mmdl = concatMap (getFixity mmdl)
 
 getFixity :: Maybe S.ModuleName -> Decl l -> [Fixity]
-getFixity mmdl (InfixDecl _ a mp ops) = let p = maybe 9 id mp in map (Fixity (sAssoc a) p) (concatMap g ops)
+getFixity mmdl d =
+  case d of
+    InfixDecl _ a mp ops  -> let p = fromMaybe 9 mp
+                              in map (Fixity (sAssoc a) p) (concatMap g ops)
+    ClassDecl _ _ _ _ cds -> maybe [] (concatMap getClassFixity) cds
+    _ -> []
   where g (VarOp _ x) = f $ sName x
         g (ConOp _ x) = f $ sName x
         f x = case mmdl of
               Nothing -> [            S.UnQual x]
               Just m  -> [S.Qual m x, S.UnQual x]
-getFixity _ _ = []
+        getClassFixity (ClsDecl _ cd) = getFixity mmdl cd
+        getClassFixity _              = []
+
+getBindFixities :: Binds l -> [Fixity]
+getBindFixities bs = case bs of
+                        BDecls _ ds -> getFixities Nothing ds
+                        _           -> []
 
 instance AppFixity Annotation where
     applyFixities fixs ann' = case ann' of
@@ -181,9 +193,10 @@ instance AppFixity InstDecl where
 
 instance AppFixity Match where
     applyFixities fixs match = case match of
-        Match l n ps rhs bs -> liftM3 (Match l n) (mapM fix ps) (fix rhs) (mapM fix bs)
-        InfixMatch l a n ps rhs bs -> liftM4 (flip (InfixMatch l) n) (fix a) (mapM fix ps) (fix rhs) (mapM fix bs)
-      where fix x = applyFixities fixs x
+        Match l n ps rhs bs -> liftM3 (Match l n) (mapM (fix bs) ps) (fix bs rhs) (mapM (fix bs) bs)
+        InfixMatch l a n ps rhs bs -> liftM4 (flip (InfixMatch l) n) (fix bs a) (mapM (fix bs) ps) (fix bs rhs) (mapM (fix bs) bs)
+      where fix bs x = applyFixities fixs' x
+             where fixs' = fixs ++ maybe [] getBindFixities bs
 
 instance AppFixity Rhs where
     applyFixities fixs rhs = case rhs of
@@ -276,7 +289,9 @@ leafFix fixs e' = case e' of
     App l e1 e2               -> liftM2 (App l) (fix e1) (fix e2)
     NegApp l e                -> liftM (NegApp l) $ fix e
     Lambda l pats e           -> liftM2 (Lambda l) (mapM fix pats) $ fix e
-    Let l bs e                -> liftM2 (Let l) (fix bs) $ fix e
+    Let l bs e                ->
+        let extraFix x = applyFixities (fixs ++ getBindFixities bs) x
+         in liftM2 (Let l) (extraFix bs) $ extraFix e
     If l e a b                -> liftM3 (If l) (fix e) (fix a) (fix b)
     MultiIf l alts            -> liftM (MultiIf l) (mapM fix alts)
     Case l e alts             -> liftM2 (Case l) (fix e) $ mapM fix alts
