@@ -40,7 +40,7 @@ sModule md = case md of
         let loc = getPointLoc l
          in S.Module loc (sModuleName mn) (map sModulePragma oss)
                       Nothing
-                      (Just [S.EVar S.NoNamespace $ S.UnQual $ S.Ident "page"])
+                      (Just [S.EVar $ S.UnQual $ S.Ident "page"])
                         []
                         [pageFun loc $ S.XTag loc (sXName xn) (map sXAttr attrs) (fmap sExp mat) (map sExp es)]
     XmlHybrid l mmh oss ids ds xn attrs mat es  ->
@@ -50,7 +50,7 @@ sModule md = case md of
          in S.Module loc1 mn (map sModulePragma oss) mwt mes (map sImportDecl ids)
                 (map sDecl ds ++ [pageFun loc2 $ S.XTag loc2 (sXName xn) (map sXAttr attrs) (fmap sExp mat) (map sExp es)])
   where pageFun :: SrcLoc -> S.Exp -> S.Decl
-        pageFun loc e = S.PatBind loc namePat rhs (S.BDecls [])
+        pageFun loc e = S.PatBind loc namePat rhs Nothing
             where namePat = S.PVar $ S.Ident "page"
                   rhs = S.UnGuardedRhs e
 
@@ -96,9 +96,11 @@ sDecl decl = case decl of
      DefaultDecl  l ts          -> S.DefaultDecl (getPointLoc l) (map sType ts)
      SpliceDecl   l sp          -> S.SpliceDecl (getPointLoc l) (sExp sp)
      TypeSig      l ns t        -> S.TypeSig (getPointLoc l) (map sName ns) (sType t)
+     PatSynSig    l n mtvs c1 c2 t ->
+        S.PatSynSig (getPointLoc l) (sName n) (fmap (map sTyVarBind) mtvs) (maybe [] sContext c1) (maybe [] sContext c2) (sType t)
      FunBind      _ ms          -> S.FunBind (map sMatch ms)
      PatBind      l p rhs mbs    ->
-        S.PatBind (getPointLoc l) (sPat p) (sRhs rhs) (maybe (S.BDecls []) sBinds mbs)
+        S.PatBind (getPointLoc l) (sPat p) (sRhs rhs) (sBinds `fmap` mbs)
      ForImp       l cc msaf mstr n t    ->
         S.ForImp (getPointLoc l) (sCallConv cc) (maybe (S.PlaySafe False) sSafety msaf) (fromMaybe "" mstr) (sName n) (sType t)
      ForExp       l cc      mstr n t    ->
@@ -118,6 +120,15 @@ sDecl decl = case decl of
         S.AnnPragma (getPointLoc l) (sAnnotation ann')
      MinimalPragma    l b           ->
         S.MinimalPragma (getPointLoc l) (fmap sBooleanFormula b)
+     RoleAnnotDecl    l qn rs        ->
+      S.RoleAnnotDecl (getPointLoc l) (sQName qn) (map sRole rs)
+     PatSyn l p r t -> S.PatSyn (getPointLoc l) (sPat p) (sPat r) (sPatSyn t)
+
+sPatSyn :: SrcInfo l => PatternSynDirection l -> S.PatternSynDirection
+sPatSyn p = case p of
+      Unidirectional             -> S.Unidirectional
+      ImplicitBidirectional      -> S.ImplicitBidirectional
+      ExplicitBidirectional _ ms -> S.ExplicitBidirectional (map sDecl ms)
 
 sTypeEqn :: SrcInfo l => TypeEqn l -> S.TypeEqn
 sTypeEqn (TypeEqn _ a b) = S.TypeEqn (sType a) (sType b)
@@ -135,6 +146,14 @@ sBooleanFormula b' = case b' of
     OrFormula _  ns  -> S.OrFormula $ map sBooleanFormula ns
     ParenFormula _ b -> S.ParenFormula (sBooleanFormula b)
 
+sRole :: Role l -> S.Role
+sRole r =
+  case r of
+    Nominal _          -> S.Nominal
+    Representational _ -> S.Representational
+    Phantom _          -> S.Phantom
+    RoleWildcard _     -> S.RoleWildcard
+
 sModuleName :: ModuleName l -> S.ModuleName
 sModuleName (ModuleName _ str)  = S.ModuleName str
 
@@ -175,7 +194,7 @@ sCName (ConName _ n) = S.ConName (sName n)
 
 sModuleHead :: Maybe (ModuleHead l) -> (S.ModuleName, Maybe S.WarningText, Maybe [S.ExportSpec])
 sModuleHead mmh = case mmh of
-    Nothing -> (S.main_mod, Nothing, Just [S.EVar S.NoNamespace (S.UnQual S.main_name)])
+    Nothing -> (S.main_mod, Nothing, Just [S.EVar (S.UnQual S.main_name)])
     Just (ModuleHead _ mn mwt mel) -> (sModuleName mn, fmap sWarningText mwt, fmap sExportSpecList mel)
 
 sExportSpecList :: ExportSpecList l -> [S.ExportSpec]
@@ -183,8 +202,8 @@ sExportSpecList (ExportSpecList _ ess) = map sExportSpec ess
 
 sExportSpec :: ExportSpec l -> S.ExportSpec
 sExportSpec es = case es of
-    EVar _ n qn         -> S.EVar (sNamespace n) (sQName qn)
-    EAbs _ qn           -> S.EAbs (sQName qn)
+    EVar _ qn           -> S.EVar (sQName qn)
+    EAbs _ n qn         -> S.EAbs (sNamespace n)(sQName qn)
     EThingAll _ qn      -> S.EThingAll (sQName qn)
     EThingWith _ qn cns -> S.EThingWith (sQName qn) (map sCName cns)
     EModuleContents _ mn    -> S.EModuleContents (sModuleName mn)
@@ -200,11 +219,12 @@ sNamespace :: Namespace l -> S.Namespace
 sNamespace n = case n of
                 NoNamespace _   -> S.NoNamespace
                 TypeNamespace _ -> S.TypeNamespace
+                PatternNamespace _ -> S.PatternNamespace
 
 sImportSpec :: ImportSpec l -> S.ImportSpec
 sImportSpec is = case is of
-    IVar _ ns n          -> S.IVar (sNamespace ns) (sName n)
-    IAbs _ n            -> S.IAbs (sName n)
+    IVar _ n            -> S.IVar (sName n)
+    IAbs _ ns n         -> S.IAbs (sNamespace ns) (sName n)
     IThingAll _ n       -> S.IThingAll (sName n)
     IThingWith _ n cns  -> S.IThingWith (sName n) (map sCName cns)
 
@@ -256,9 +276,9 @@ sIPBind (IPBind l ipn e) = S.IPBind (getPointLoc l) (sIPName ipn) (sExp e)
 
 sMatch :: SrcInfo loc => Match loc -> S.Match
 sMatch (Match l n ps rhs mwhere) =
-    S.Match (getPointLoc l) (sName n) (map sPat ps) Nothing (sRhs rhs) (maybe (S.BDecls []) sBinds mwhere)
+    S.Match (getPointLoc l) (sName n) (map sPat ps) Nothing (sRhs rhs) (sBinds `fmap` mwhere)
 sMatch (InfixMatch l pa n pbs rhs mwhere) =
-    S.Match (getPointLoc l) (sName n) (map sPat (pa:pbs)) Nothing (sRhs rhs) (maybe (S.BDecls []) sBinds mwhere)
+    S.Match (getPointLoc l) (sName n) (map sPat (pa:pbs)) Nothing (sRhs rhs) (sBinds `fmap` mwhere)
 
 sQualConDecl :: SrcInfo loc => QualConDecl loc -> S.QualConDecl
 sQualConDecl (QualConDecl l mtvs mctxt cd) =
@@ -332,14 +352,15 @@ sType t' = case t' of
     TyEquals _ t1 t2            -> S.TyEquals (sType t1) (sType t2)
     TySplice _ s                -> S.TySplice (sSplice s)
     TyBang _ b t                -> S.TyBang (sBangType b) (sType t)
+    TyWildCard _ mn             -> S.TyWildCard (sName `fmap` mn)
 
-sPromoted :: Promoted l -> S.Promoted
+sPromoted :: SrcInfo l => Promoted l -> S.Promoted
 sPromoted p = case p of
     PromotedInteger _ n _ -> S.PromotedInteger n
     PromotedString _ s _ -> S.PromotedString s
     PromotedCon _ b qn -> S.PromotedCon b (sQName qn)
-    PromotedList _ b ps -> S.PromotedList b (map sPromoted ps)
-    PromotedTuple _ ps -> S.PromotedTuple (map sPromoted ps)
+    PromotedList _ b ps -> S.PromotedList b (map sType ps)
+    PromotedTuple _ ps -> S.PromotedTuple (map sType ps)
     PromotedUnit _ -> S.PromotedUnit
 
 
@@ -350,13 +371,12 @@ sTyVarBind (UnkindedVar _ n) = S.UnkindedVar (sName n)
 sKind :: Kind l -> S.Kind
 sKind k' = case k' of
     KindStar  _     -> S.KindStar
-    KindBang  _     -> S.KindBang
     KindFn _ k1 k2  -> S.KindFn (sKind k1) (sKind k2)
     KindParen _ k   -> S.KindParen (sKind k)
     KindVar _ n     -> S.KindVar (sQName n)
     KindApp _ k1 k2 -> S.KindApp (sKind k1) (sKind k2)
     KindTuple _ ks  -> S.KindTuple (map sKind ks)
-    KindList  _ ks  -> S.KindList  (map sKind ks)
+    KindList  _ ks  -> S.KindList  (sKind ks)
 
 sFunDep :: FunDep l -> S.FunDep
 sFunDep (FunDep _ as bs) = S.FunDep (map sName as) (map sName bs)
@@ -370,11 +390,12 @@ sContext ctxt = case ctxt of
 sAsst :: SrcInfo l => Asst l -> S.Asst
 sAsst asst = case asst of
     ClassA _ qn ts      -> S.ClassA (sQName qn) (map sType ts)
-    VarA _ n            -> S.VarA (sName n)
+    AppA _ n ns         -> S.AppA (sName n) (map sType ns)
     InfixA _ ta qn tb   -> S.InfixA (sType ta) (sQName qn) (sType tb)
     IParam _ ipn t      -> S.IParam (sIPName ipn) (sType t)
     EqualP _ t1 t2      -> S.EqualP (sType t1) (sType t2)
     ParenA _ a          -> S.ParenA (sAsst a)
+    WildCardA _ mn      -> S.WildCardA (sName `fmap` mn)
 
 sLiteral :: Literal l -> S.Literal
 sLiteral lit = case lit of
@@ -448,6 +469,7 @@ sExp e' = case e' of
     LeftArrHighApp  _ e1 e2 -> S.LeftArrHighApp (sExp e1) (sExp e2)
     RightArrHighApp _ e1 e2 -> S.RightArrHighApp (sExp e1) (sExp e2)
     LCase _ alts -> S.LCase (map sAlt alts)
+    ExprHole _              -> S.ExprHole
 
 
 sXName :: XName l -> S.XName
@@ -474,13 +496,14 @@ sSafety (PlaySafe _ b) = S.PlaySafe b
 sSafety (PlayInterruptible _) = S.PlayInterruptible
 
 sCallConv :: CallConv l -> S.CallConv
-sCallConv (StdCall   _) = S.StdCall
-sCallConv (CCall     _) = S.CCall
-sCallConv (CPlusPlus _) = S.CPlusPlus
-sCallConv (DotNet    _) = S.DotNet
-sCallConv (Jvm       _) = S.Jvm
-sCallConv (Js        _) = S.Js
-sCallConv (CApi      _) = S.CApi
+sCallConv (StdCall    _) = S.StdCall
+sCallConv (CCall      _) = S.CCall
+sCallConv (CPlusPlus  _) = S.CPlusPlus
+sCallConv (DotNet     _) = S.DotNet
+sCallConv (Jvm        _) = S.Jvm
+sCallConv (Js         _) = S.Js
+sCallConv (JavaScript _) = S.JavaScript
+sCallConv (CApi       _) = S.CApi
 
 sModulePragma :: SrcInfo loc => ModulePragma loc -> S.ModulePragma
 sModulePragma pr = case pr of
@@ -588,4 +611,4 @@ sFieldUpdate fu = case fu of
     FieldWildcard _         -> S.FieldWildcard
 
 sAlt :: SrcInfo loc => Alt loc -> S.Alt
-sAlt (Alt l p galts mbs) = S.Alt (getPointLoc l) (sPat p) (sRhs galts) (maybe (S.BDecls []) sBinds mbs)
+sAlt (Alt l p galts mbs) = S.Alt (getPointLoc l) (sPat p) (sRhs galts) (sBinds `fmap` mbs)
