@@ -46,7 +46,7 @@ module Language.Haskell.Exts.Annotated.Syntax (
     Module(..), ModuleHead(..), WarningText(..), ExportSpecList(..), ExportSpec(..),
     ImportDecl(..), ImportSpecList(..), ImportSpec(..), Assoc(..), Namespace(..),
     -- * Declarations
-    Decl(..), DeclHead(..), InstRule(..), InstHead(..), Binds(..), IPBind(..),
+    Decl(..), DeclHead(..), InstRule(..), InstHead(..), Binds(..), IPBind(..), PatternSynDirection(..),
     -- ** Type classes and instances
     ClassDecl(..), InstDecl(..), Deriving(..),
     -- ** Data type declarations
@@ -67,7 +67,7 @@ module Language.Haskell.Exts.Annotated.Syntax (
     Literal(..), Sign(..),
     -- * Variables, Constructors and Operators
     ModuleName(..), QName(..), Name(..), QOp(..), Op(..),
-    SpecialCon(..), CName(..), IPName(..), XName(..),
+    SpecialCon(..), CName(..), IPName(..), XName(..), Role(..),
 
     -- * Template Haskell
     Bracket(..), Splice(..),
@@ -93,7 +93,7 @@ module Language.Haskell.Exts.Annotated.Syntax (
     as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name,
     export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name,
     stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name,
-    capi_name, forall_name, family_name,
+    javascript_name, capi_name, forall_name, family_name,
     -- ** Type constructors
     unit_tycon_name, fun_tycon_name, list_tycon_name, tuple_tycon_name, unboxed_singleton_tycon_name,
     unit_tycon, fun_tycon, list_tycon, tuple_tycon, unboxed_singleton_tycon,
@@ -191,8 +191,8 @@ data ExportSpecList l
 
 -- | An item in a module's export specification.
 data ExportSpec l
-     = EVar l (Namespace l) (QName l)   -- ^ variable.
-     | EAbs l (QName l)                 -- ^ @T@:
+     = EVar l (QName l)                 -- ^ variable.
+     | EAbs l (Namespace l) (QName l)   -- ^ @T@:
                                         --   a class or datatype exported abstractly,
                                         --   or a type synonym.
      | EThingAll l (QName l)            -- ^ @T(..)@:
@@ -206,7 +206,7 @@ data ExportSpec l
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
 -- | Namespaces for imports/exports.
-data Namespace l = NoNamespace l | TypeNamespace l
+data Namespace l = NoNamespace l | TypeNamespace l | PatternNamespace l
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
 -- | An import declaration.
@@ -234,8 +234,8 @@ data ImportSpecList l
 -- | An import specification, representing a single explicit item imported
 --   (or hidden) from a module.
 data ImportSpec l
-     = IVar l (Namespace l) (Name l)    -- ^ variable
-     | IAbs l (Name l)                  -- ^ @T@:
+     = IVar l (Name l)                  -- ^ variable
+     | IAbs l (Namespace l) (Name l)    -- ^ @T@:
                                         --   the name of a class, datatype or type synonym.
      | IThingAll l (Name l)             -- ^ @T(..)@:
                                         --   a class imported with all of its methods, or
@@ -286,10 +286,14 @@ data Decl l
      -- ^ A Template Haskell splicing declaration
      | TypeSig      l [Name l] (Type l)
      -- ^ A type signature declaration
+     | PatSynSig    l (Name l) (Maybe [TyVarBind l]) (Maybe (Context l)) (Maybe (Context l)) (Type l)
+     -- ^ A pattern synonym signature declation
      | FunBind      l [Match l]
      -- ^ A set of function binding clauses
      | PatBind      l (Pat l) (Rhs l) {-where-} (Maybe (Binds l))
      -- ^ A pattern binding
+     | PatSyn l (Pat l) (Pat l) (PatternSynDirection l)
+     -- ^ A pattern synonym binding
      | ForImp       l (CallConv l) (Maybe (Safety l)) (Maybe String) (Name l) (Type l)
      -- ^ A foreign import declaration
      | ForExp       l (CallConv l)                    (Maybe String) (Name l) (Type l)
@@ -314,8 +318,16 @@ data Decl l
      -- ^ An ANN pragma
      | MinimalPragma    l (Maybe (BooleanFormula l))
      -- ^ A MINIMAL pragma
+     | RoleAnnotDecl    l (QName l) [Role l]
+     -- ^ A role annotation
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
+data  PatternSynDirection l =
+      Unidirectional -- ^ A unidirectional pattern synonym with "<-"
+    | ImplicitBidirectional  -- ^ A bidirectional pattern synonym with "="
+    | ExplicitBidirectional l [Decl l]  -- ^ A birectional pattern synonym with the construction specified.
+    deriving (Eq, Ord, Show, Data, Typeable, Foldable, Traversable, Functor, Generic)
+
 -- | A type equation as found in closed type families.
 data TypeEqn l = TypeEqn l (Type l) (Type l) deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
@@ -337,6 +349,13 @@ data BooleanFormula l
     | ParenFormula l (BooleanFormula l)  -- ^ Parenthesized boolean formulas.
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
+data Role l
+  = Nominal l
+  | Representational l
+  | Phantom l
+  | RoleWildcard l
+  deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
+
 -- | A flag stating whether a declaration is a data or newtype declaration.
 data DataOrNew l = DataType l | NewType l
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
@@ -577,6 +596,7 @@ data Type l
      | TyEquals l (Type l) (Type l)             -- ^ type equality predicate enabled by ConstraintKinds
      | TySplice l (Splice l)                    -- ^ template haskell splice type
      | TyBang l (BangType l) (Type l)           -- ^ Strict type marked with \"@!@\" or type marked with UNPACK pragma.
+     | TyWildCard l (Maybe (Name l))            -- ^ Either an anonymous of named type wildcard
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
 -- | Bools here are True if there was a leading quote which may be
@@ -585,8 +605,8 @@ data Promoted l
         = PromotedInteger l Integer String -- ^ parsed value and raw string
         | PromotedString l String String -- ^ parsed value and raw string
         | PromotedCon l Bool (QName l)
-        | PromotedList l Bool [Promoted l]
-        | PromotedTuple l [Promoted l]
+        | PromotedList l Bool [Type l]
+        | PromotedTuple l [Type l]
         | PromotedUnit l
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
@@ -603,13 +623,12 @@ data TyVarBind l
 -- | An explicit kind annotation.
 data Kind l
     = KindStar  l                    -- ^ @*@, the kind of types
-    | KindBang  l                    -- ^ @!@, the kind of unboxed types
     | KindFn    l (Kind l) (Kind l)  -- ^ @->@, the kind of a type constructor
     | KindParen l (Kind l)           -- ^ a parenthesised kind
     | KindVar   l (QName l)          -- ^ @k@, a kind variable (-XPolyKinds)
     | KindApp   l (Kind l) (Kind l)  -- ^ @k1 k2@
     | KindTuple l [Kind l]           -- ^ @'(k1,k2,k3)@, a promoted tuple
-    | KindList  l [Kind l]           -- ^ @'[k1,k2,k3]@, a promoted list literal
+    | KindList  l (Kind l)           -- ^ @'[k1]@, a promoted list literal
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
 
@@ -632,11 +651,12 @@ data Context l
 --   Also extended with support for implicit parameters and equality constraints.
 data Asst l
         = ClassA l (QName l) [Type l]           -- ^ ordinary class assertion
-        | VarA l (Name l)                       -- ^ constraint kind assertion, @Dict :: cxt => Dict cxt@
+        | AppA l (Name l) [Type l]              -- ^ constraint kind assertion, @Dict :: cxt a => Dict cxt@
         | InfixA l (Type l) (QName l) (Type l)  -- ^ class assertion where the class name is given infix
         | IParam l (IPName l) (Type l)          -- ^ implicit parameter assertion
         | EqualP l (Type l) (Type l)            -- ^ type equality constraint
         | ParenA l (Asst l)                     -- ^ parenthesised class assertion
+        | WildCardA l (Maybe (Name l))          -- ^ Context Wildcard
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
 -- | /literal/
@@ -739,6 +759,9 @@ data Exp l
 
 -- LambdaCase
     | LCase l [Alt l]                       -- ^ @\case@ /alts/
+
+-- Holes
+    | ExprHole l                            -- ^ Expression hole
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
 -- | The name of an xml element or attribute,
@@ -781,6 +804,7 @@ data CallConv l
     | DotNet l
     | Jvm l
     | Js l
+    | JavaScript l
     | CApi l
   deriving (Eq,Ord,Show,Typeable,Data,Foldable,Traversable,Functor,Generic)
 
@@ -970,7 +994,8 @@ star_name      l = Symbol l "*"
 
 export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name,
   stdcall_name, ccall_name, cplusplus_name, dotnet_name,
-  jvm_name, js_name, capi_name, forall_name, family_name :: l -> Name l
+  jvm_name, js_name, javascript_name, capi_name, forall_name,
+  family_name :: l -> Name l
 export_name     l = Ident l "export"
 safe_name       l = Ident l "safe"
 unsafe_name     l = Ident l "unsafe"
@@ -982,6 +1007,7 @@ cplusplus_name  l = Ident l "cplusplus"
 dotnet_name     l = Ident l "dotnet"
 jvm_name        l = Ident l "jvm"
 js_name         l = Ident l "js"
+javascript_name l = Ident l "javascript"
 capi_name       l = Ident l "capi"
 forall_name     l = Ident l "forall"
 family_name     l = Ident l "family"
@@ -1097,14 +1123,14 @@ instance Annotated ExportSpecList where
 
 instance Annotated ExportSpec where
     ann es = case es of
-        EVar l _ _          -> l
-        EAbs l _            -> l
+        EVar l _            -> l
+        EAbs l _ _          -> l
         EThingAll l _       -> l
         EThingWith l _ _    -> l
         EModuleContents l _ -> l
     amap f es = case es of
-        EVar l t qn     -> EVar (f l) t qn
-        EAbs l qn       -> EAbs (f l) qn
+        EVar l qn     -> EVar (f l) qn
+        EAbs l n qn       -> EAbs (f l) n qn
         EThingAll l qn  -> EThingAll (f l) qn
         EThingWith l qn cns -> EThingWith (f l) qn cns
         EModuleContents l mn    -> EModuleContents (f l) mn
@@ -1113,9 +1139,11 @@ instance Annotated Namespace where
     ann es = case es of
         NoNamespace l   -> l
         TypeNamespace l -> l
+        PatternNamespace l -> l
     amap f es = case es of
         NoNamespace l   -> NoNamespace (f l)
         TypeNamespace l -> TypeNamespace (f l)
+        PatternNamespace l -> PatternNamespace (f l)
 
 instance Annotated ImportDecl where
     ann (ImportDecl l _ _ _ _ _ _ _) = l
@@ -1128,13 +1156,13 @@ instance Annotated ImportSpecList where
 
 instance Annotated ImportSpec where
     ann is = case is of
-        IVar l _ _       -> l
-        IAbs l _         -> l
+        IVar l _         -> l
+        IAbs l _ _       -> l
         IThingAll l _    -> l
         IThingWith l _ _ -> l
     amap f is = case is of
-        IVar l t n      -> IVar (f l) t n
-        IAbs l n        -> IAbs (f l) n
+        IVar l n        -> IVar (f l) n
+        IAbs l ns n     -> IAbs (f l) ns n
         IThingAll l n   -> IThingAll (f l) n
         IThingWith l n cns  -> IThingWith (f l) n cns
 
@@ -1170,6 +1198,7 @@ instance Annotated Decl where
         DefaultDecl  l _                -> l
         SpliceDecl   l _                -> l
         TypeSig      l _ _              -> l
+        PatSynSig    l _ _ _ _ _        -> l
         FunBind      l _                -> l
         PatBind      l _ _ _            -> l
         ForImp       l _ _ _ _ _        -> l
@@ -1184,6 +1213,8 @@ instance Annotated Decl where
         InstSig          l _            -> l
         AnnPragma        l _            -> l
         MinimalPragma    l _            -> l
+        RoleAnnotDecl    l _ _          -> l
+        PatSyn           l _ _ _        -> l
     amap f decl = case decl of
         TypeDecl     l dh t      -> TypeDecl    (f l) dh t
         TypeFamDecl  l dh mk     -> TypeFamDecl (f l) dh mk
@@ -1203,6 +1234,7 @@ instance Annotated Decl where
         DefaultDecl  l ts                -> DefaultDecl (f l) ts
         SpliceDecl   l sp                -> SpliceDecl (f l) sp
         TypeSig      l ns t              -> TypeSig (f l) ns t
+        PatSynSig    l n dh c1 c2 t      -> PatSynSig (f l) n dh c1 c2 t
         FunBind      l ms                -> FunBind (f l) ms
         PatBind      l p rhs bs          -> PatBind (f l) p rhs bs
         ForImp       l cc msf s n t      -> ForImp (f l) cc msf s n t
@@ -1217,6 +1249,20 @@ instance Annotated Decl where
         InstSig          l ih            -> InstSig (f l) ih
         AnnPragma        l ann'          -> AnnPragma (f l) ann'
         MinimalPragma    l b             -> MinimalPragma (f l) b
+        RoleAnnotDecl    l t rs          -> RoleAnnotDecl (f l) t rs
+        PatSyn           l p r d         -> PatSyn (f l) p r d
+
+instance Annotated Role where
+    ann r = case r of
+      RoleWildcard l -> l
+      Representational l -> l
+      Phantom l -> l
+      Nominal l -> l
+    amap f r = case r of
+      RoleWildcard l -> RoleWildcard (f l)
+      Representational l -> Representational (f l)
+      Phantom l -> Phantom (f l)
+      Nominal l -> Nominal (f l)
 
 instance Annotated Annotation where
     ann (Ann     l _ _) = l
@@ -1362,6 +1408,7 @@ instance Annotated Type where
       TyEquals l _ _                -> l
       TySplice l _                  -> l
       TyBang l _ _                  -> l
+      TyWildCard l _                -> l
     amap f t1 = case t1 of
       TyForall l mtvs mcx t         -> TyForall (f l) mtvs mcx t
       TyFun   l t1' t2              -> TyFun (f l) t1' t2
@@ -1378,6 +1425,7 @@ instance Annotated Type where
       TyEquals l a b                -> TyEquals (f l) a b
       TySplice l s                  -> TySplice (f l) s
       TyBang l b t                  -> TyBang (f l) b t
+      TyWildCard l n                -> TyWildCard (f l) n
 
 instance Annotated TyVarBind where
     ann (KindedVar   l _ _) = l
@@ -1387,7 +1435,6 @@ instance Annotated TyVarBind where
 
 instance Annotated Kind where
     ann (KindStar l) = l
-    ann (KindBang l) = l
     ann (KindFn   l _ _) = l
     ann (KindParen l _)  = l
     ann (KindVar l _)    = l
@@ -1395,7 +1442,6 @@ instance Annotated Kind where
     ann (KindTuple l _)  = l
     ann (KindList  l _)  = l
     amap f (KindStar l) = KindStar (f l)
-    amap f (KindBang l) = KindBang (f l)
     amap f (KindFn   l k1 k2) = KindFn (f l) k1 k2
     amap f (KindParen l k) = KindParen (f l) k
     amap f (KindVar l n) = KindVar (f l) n
@@ -1418,18 +1464,20 @@ instance Annotated Context where
 instance Annotated Asst where
     ann asst = case asst of
         ClassA l _ _     -> l
-        VarA l _         -> l
+        AppA l _ _       -> l
         InfixA l _ _ _   -> l
         IParam l _ _     -> l
         EqualP l _ _     -> l
         ParenA l _       -> l
+        WildCardA l _    -> l
     amap f asst = case asst of
         ClassA l qn ts      -> ClassA (f l) qn ts
-        VarA   l n          -> VarA   (f l) n
+        AppA   l n ns       -> AppA   (f l) n ns
         InfixA l ta qn tb   -> InfixA (f l) ta qn tb
         IParam l ipn t      -> IParam (f l) ipn t
         EqualP l t1 t2      -> EqualP (f l) t1 t2
         ParenA l a          -> ParenA (f l) a
+        WildCardA l mn      -> WildCardA (f l) mn
 
 instance Annotated Literal where
     ann lit = case lit of
@@ -1509,6 +1557,7 @@ instance Annotated Exp where
         RightArrHighApp l _ _  -> l
 
         LCase l _              -> l
+        ExprHole l             -> l
 
     amap f e1 = case e1 of
         Var l qn        -> Var (f l) qn
@@ -1567,6 +1616,7 @@ instance Annotated Exp where
 
         LCase l alts -> LCase (f l) alts
         MultiIf l alts -> MultiIf (f l) alts
+        ExprHole l      -> ExprHole (f l)
 
 
 instance Annotated XName where
@@ -1607,6 +1657,7 @@ instance Annotated CallConv where
     ann (DotNet l) = l
     ann (Jvm l) = l
     ann (Js l) = l
+    ann (JavaScript l) = l
     ann (CApi l) = l
     amap = fmap
 
