@@ -31,7 +31,7 @@ module Language.Haskell.Exts.Extension (
     parseExtension, prettyExtension,
 
     -- * Extension groups
-    ghcDefault, glasgowExts, 
+    ghcDefault, glasgowExts,
     knownExtensions, deprecatedExtensions,
 
     -- * Semantics of extensions applied to languages
@@ -41,7 +41,7 @@ module Language.Haskell.Exts.Extension (
 
 import Control.Applicative ((<$>), (<|>))
 import Data.Array (Array, accumArray, bounds, Ix(inRange), (!))
-import Data.List (nub, delete)
+import Data.List (nub)
 import Data.Maybe (fromMaybe)
 import Data.Data
 
@@ -269,7 +269,7 @@ data KnownExtension =
   | ExtensibleRecords
 
   -- | [Hugs &#xa7; 7.2] Enable type synonyms which are transparent in
-  -- some definitions and opaque elsewhere, as a way of implementing 
+  -- some definitions and opaque elsewhere, as a way of implementing
   -- abstract datatypes.
   | RestrictedTypeSynonyms
 
@@ -504,7 +504,21 @@ data KnownExtension =
   -- Constraint can occur before the arrow.
   | ConstraintKinds
 
+  -- | [GHC &#xa7; 7.25.3] Allow role annotations.
+  | RoleAnnotations
+  -- | [GHC &#xa7; 7.3.8] Enable giving names to parametrized pattern
+  -- schemes
+  | PatternSynonyms
 
+  -- | [GHC &#xa7; 7.15] Allow partial specified type signatures.
+  -- Note that this extension does not affect parsing. It only affects whether the
+  -- program is able to be run with partial signatures.
+  | PartialTypeSignatures
+
+  -- | [GHC &#xa7; 7.15.1.2] This extension treats type variables which
+  -- start with an underscore as wildcards. For example, `foo :: _x` is
+  -- equivalent to `foo :: _`.
+  | NamedWildCards
   deriving (Show, Read, Eq, Ord, Enum, Bounded, Data, Typeable)
 
 -- | Certain extensions imply other extensions, and this function
@@ -513,12 +527,12 @@ data KnownExtension =
 --   The returned value is the transitive closure of implied
 --   extensions.
 {-impliesExts :: [Extension] -> [Extension]
-impliesExts exts = 
+impliesExts exts =
     let posExts = [ ke | EnableExtension  ke <- exts ]
         negExts = [ ke | DisableExtension ke <- exts ]
 
         implExts = impliesKnownExts posExts
-     in 
+     in
 -}
 impliesExts :: [KnownExtension] -> [KnownExtension]
 impliesExts = go
@@ -538,6 +552,7 @@ impliesExts = go
                     LiberalTypeSynonyms -> [ExplicitForAll]
                     ExistentialQuantification -> [ExplicitForAll]
                     ImpredicativeTypes   -> [ExplicitForAll]
+                    PolyKinds           -> [KindSignatures]
                     -- Deprecations
                     RecordPuns          -> [NamedFieldPuns]
                     PatternSignatures   -> [ScopedTypeVariables]
@@ -654,8 +669,8 @@ readMay s = case [x | (x,t) <- reads s, ("","") <- lex t] of
                 _ -> Nothing
 
 {-------------------------------------------
- -- Transform a 'Language', and possibly a modifying set of'Extension's, into a list 
- -- of 'KnownExtension's, to be interpreted as modifying the language you get 
+ -- Transform a 'Language', and possibly a modifying set of'Extension's, into a list
+ -- of 'KnownExtension's, to be interpreted as modifying the language you get
  -- when all known extensions are disabled.
  -- Extensions are interpreted in a right-biased fashion, so the last instance
  -- of an occurence of 'EnableExtension' or 'DisableExtension' for a given
@@ -669,21 +684,21 @@ toExtensionList lang exts' =
                     Haskell2010 -> [DoAndIfThenElse
                                    , PatternGuards
                                    , ForeignFunctionInterface
-                                   , EmptyDataDecls 
+                                   , EmptyDataDecls
                                    ] ++ allLangDefault
                     HaskellAllDisabled -> []
-                    UnknownLanguage s -> 
+                    UnknownLanguage s ->
                         error $ "toExtensionList: Unknown language " ++ s
 {-
         addExts = [ ke | EnableExtension  ke <- exts ]
         remExts = [ ke | DisableExtension ke <- exts ]
      in impliesExts $ nub $ (langKes ++ addExts) \\ remExts
 -}
-  in impliesExts $ go langKes exts'
+  in impliesExts $ nub $ go langKes exts'
     where go :: [KnownExtension] -> [Extension] -> [KnownExtension]
           go acc [] = acc
-          go acc (DisableExtension x : exts) = go (nub (delete x acc)) exts
-          go acc (EnableExtension  x : exts) = go (nub (x : acc))      exts
+          go acc (DisableExtension x : exts) = go (filter (/= x) acc) exts
+          go acc (EnableExtension  x : exts) = go (x : acc)           exts
           -- We just throw away UnknownExtensions
           go acc (_ : exts) = go acc exts
 
