@@ -93,10 +93,10 @@ data Token
         | Minus
         | Exclamation
         | Star
-        | LeftArrowTail         -- >-
-        | RightArrowTail        -- -<
-        | LeftDblArrowTail      -- >>-
-        | RightDblArrowTail     -- -<<
+        | LeftArrowTail         -- -<
+        | RightArrowTail        -- >-
+        | LeftDblArrowTail      -- -<<
+        | RightDblArrowTail     -- >>-
 
 -- Template Haskell
         | THExpQuote            -- [| or [e|
@@ -181,11 +181,13 @@ data Token
         | KW_Of
         | KW_Proc       -- arrows
         | KW_Rec        -- arrows
+        | KW_Role
         | KW_Then
         | KW_Type
         | KW_Using      -- transform list comprehensions
         | KW_Where
         | KW_Qualified
+        | KW_Pattern
 
                 -- FFI
         | KW_Foreign
@@ -200,6 +202,7 @@ data Token
         | KW_DotNet
         | KW_Jvm
         | KW_Js
+        | KW_JavaScript
         | KW_CApi
 
         | EOF
@@ -281,6 +284,8 @@ reserved_ids = [
  ( "type",      (KW_Type,       Nothing) ),
  ( "using",     (KW_Using,      Just (Any [TransformListComp])) ),
  ( "where",     (KW_Where,      Nothing) ),
+ ( "role",      (KW_Role,       Just (Any [RoleAnnotations]))),
+ ( "pattern",   (KW_Pattern,    Just (Any [PatternSynonyms]))),
 
 -- FFI
  ( "foreign",   (KW_Foreign,    Just (Any [ForeignFunctionInterface])) )
@@ -305,6 +310,7 @@ special_varids = [
  ( "dotnet",        (KW_DotNet,        Just (Any [ForeignFunctionInterface])) ),
  ( "jvm",           (KW_Jvm,           Just (Any [ForeignFunctionInterface])) ),
  ( "js",            (KW_Js,            Just (Any [ForeignFunctionInterface])) ),
+ ( "javascript",    (KW_JavaScript,    Just (Any [ForeignFunctionInterface])) ),
  ( "capi",          (KW_CApi,          Just (Any [CApiFFI])) )
  ]
 
@@ -335,7 +341,7 @@ pragmas = [
  ]
 
 isIdent, isHSymbol, isPragmaChar :: Char -> Bool
-isIdent   c = isAlpha c || isDigit c || c == '\'' || c == '_'
+isIdent   c = isAlphaNum c || c == '\'' || c == '_'
 
 isHSymbol c = c `elem` ":!#%&*./?@\\-" || ((isSymbol c || isPunctuation c) && not (c `elem` "(),;[]`{}_\"'"))
 
@@ -884,17 +890,23 @@ lexPragmaStart = do
              _ -> return SPECIALISE
 
      Just (OPTIONS opt) ->     -- see, I promised we'd mask out the 'undefined'
-            case fst opt of
-             Just opt' -> do
-                rest <- lexRawPragma
-                return $ OPTIONS (Just opt', rest)
-             Nothing -> do
-                            s <- getInput
-                            case s of
-                                x:_ | isSpace x -> do
-                                    rest <- lexRawPragma
-                                    return $ OPTIONS (Nothing, rest)
-                                _  -> fail "Malformed Options pragma"
+            -- We do not want to store necessary whitespace in the datatype
+            -- but if the pragma starts with a newline then we must keep
+            -- it to differentiate the two cases.
+            let dropIfSpace (' ':xs) = xs
+                dropIfSpace xs       = xs
+             in
+              case fst opt of
+                Just opt' -> do
+                  rest <- lexRawPragma
+                  return $ OPTIONS (Just opt', dropIfSpace rest)
+                Nothing -> do
+                  s <- getInput
+                  case s of
+                    x:_ | isSpace x -> do
+                      rest <- lexRawPragma
+                      return $ OPTIONS (Nothing, dropIfSpace rest)
+                    _  -> fail "Malformed Options pragma"
      Just RULES -> do -- Rules enable ScopedTypeVariables locally.
             addExtensionL ScopedTypeVariables
             return RULES
@@ -1286,10 +1298,10 @@ showToken t = case t of
   Minus             -> "-"
   Exclamation       -> "!"
   Star              -> "*"
-  LeftArrowTail     -> ">-"
-  RightArrowTail    -> "-<"
-  LeftDblArrowTail  -> ">>-"
-  RightDblArrowTail -> "-<<"
+  LeftArrowTail     -> "-<"
+  RightArrowTail    -> ">-"
+  LeftDblArrowTail  -> "-<<"
+  RightDblArrowTail -> ">>-"
   THExpQuote        -> "[|"
   THPatQuote        -> "[p|"
   THDecQuote        -> "[d|"
@@ -1379,6 +1391,9 @@ showToken t = case t of
   KW_DotNet     -> "dotnet"
   KW_Jvm        -> "jvm"
   KW_Js         -> "js"
+  KW_JavaScript -> "javascript"
   KW_CApi       -> "capi"
+  KW_Role       -> "role"
+  KW_Pattern    -> "pattern"
 
   EOF           -> "EOF"
