@@ -15,12 +15,14 @@
 -----------------------------------------------------------------------------
 
 module Language.Haskell.Exts.ParseMonad(
+        -- * Generic Parsing
+        Parseable(..),
         -- * Parsing
         P, ParseResult(..), atSrcLoc, LexContext(..),
         ParseMode(..), defaultParseMode, fromParseResult,
         runParserWithMode, runParserWithModeComments, runParser,
         getSrcLoc, pushCurrentContext, popContext,
-        getExtensions,
+        getExtensions, getIgnoreFunctionArity,
         -- * Lexing
         Lex(runL), getInput, discard, lexNewline, lexTab, lexWhile, lexWhile_,
         alternative, checkBOL, setBOL, startToken, getOffside,
@@ -44,6 +46,25 @@ import Control.Applicative
 import Control.Monad (when, liftM, ap)
 import Data.Monoid
 
+-- | Class providing function for parsing at many different types.
+--
+--   Note that for convenience of implementation, the default methods have
+--   definitions equivalent to 'undefined'.  The minimal definition is all of
+--   the visible methods.
+class Parseable ast where
+  -- | Parse a string with default mode.
+  parse :: String -> ParseResult ast
+  parse = parseWithMode defaultParseMode
+  -- | Parse a string with an explicit 'ParseMode'.
+  parseWithMode :: ParseMode -> String -> ParseResult ast
+  parseWithMode mode = runParserWithMode mode . parser $ fixities mode
+  -- | Parse a string with an explicit 'ParseMode', returning all comments along
+  --   with the AST.
+  parseWithComments :: ParseMode -> String -> ParseResult (ast, [Comment])
+  parseWithComments mode = runParserWithModeComments mode . parser $ fixities mode
+  -- | Internal parser, used to provide default definitions for the others.
+  parser :: Maybe [Fixity] -> P ast
+
 -- | The result of a parse.
 data ParseResult a
     = ParseOk a  -- ^ The parse succeeded, yielding a value.
@@ -123,7 +144,9 @@ data ParseMode = ParseMode {
         --   from LINE pragmas in source files
         ignoreLinePragmas :: Bool,
         -- | list of fixities to be aware of
-        fixities :: Maybe [Fixity]
+        fixities :: Maybe [Fixity],
+        -- | Checks whether functions have a consistent arity
+        ignoreFunctionArity :: Bool
         }
 
 -- | Default parameters for a parse.
@@ -138,7 +161,8 @@ defaultParseMode = ParseMode {
         extensions = [],
         ignoreLanguagePragmas = False,
         ignoreLinePragmas = True,
-        fixities = Just preludeFixities
+        fixities = Just preludeFixities,
+        ignoreFunctionArity = False
         }
 
 -- Version of ParseMode used internally,
@@ -148,13 +172,14 @@ data InternalParseMode = IParseMode {
         iParseFilename :: String,
         iExtensions :: [KnownExtension],
         -- iIgnoreLanguagePragmas :: Bool,
-        iIgnoreLinePragmas :: Bool
+        iIgnoreLinePragmas :: Bool,
+        iIgnoreFunctionArity :: Bool
         -- iFixities :: Maybe [Fixity]
     }
 
 toInternalParseMode :: ParseMode -> InternalParseMode
-toInternalParseMode (ParseMode pf bLang exts _ilang iline _fx) =
-    IParseMode pf (impliesExts $ toExtensionList bLang exts) {-_ilang -} iline {- _fx -}
+toInternalParseMode (ParseMode pf bLang exts _ilang iline _fx farity) =
+    IParseMode pf (toExtensionList bLang exts) {-_ilang -} iline {- _fx -} farity
 
 
 -- | Monad for parsing
@@ -179,14 +204,14 @@ runParserWithMode :: ParseMode -> P a -> String -> ParseResult a
         srcColumn = 1
     }
 -}
-runParserWithMode mode pm s = fmap fst $ runParserWithModeComments mode pm s
+runParserWithMode mode pm = fmap fst . runParserWithModeComments mode pm
 
 runParser :: P a -> String -> ParseResult a
 runParser = runParserWithMode defaultParseMode
 
 runParserWithModeComments :: ParseMode -> P a -> String -> ParseResult (a, [Comment])
-runParserWithModeComments mode (P m) s = 
-  case m s 0 1 start ([],[],[],(False,False),[]) (toInternalParseMode mode) of
+runParserWithModeComments mode = let mode2 = toInternalParseMode mode in \(P m) s ->
+  case m s 0 1 start ([],[],[],(False,False),[]) mode2 of
     Ok (_,_,_,_,cs) a -> ParseOk (a, reverse cs)
     Failed loc msg    -> ParseFailed loc msg
     where start = SrcLoc {
@@ -292,6 +317,12 @@ pushCtxtFlag =
 pullDoStatus :: P Bool
 pullDoStatus = P $ \_i _x _y _l (s, exts, e, (d,c), cs) _m -> Ok (s,exts,e,(False,c),cs) d
 
+getIgnoreFunctionArity :: P Bool
+getIgnoreFunctionArity = P $ \_i _x _y _l s m ->
+  Ok s $ iIgnoreFunctionArity m
+
+
+
 
 ----------------------------------------------------------------------------
 -- Monad for lexical analysis:
@@ -344,9 +375,20 @@ tAB_LENGTH = 8
 -- Consume and return the largest string of characters satisfying p
 
 lexWhile :: (Char -> Bool) -> Lex a String
-lexWhile p = Lex $ \cont -> P $ \r x ->
-    let (cs,rest) = span p r in
-    runP (cont cs) rest (x + length cs)
+lexWhile p = Lex $ \cont -> P $ \rss c l ->
+  case rss of
+    [] -> runP (cont []) [] c l
+    (r:rs) ->
+      let
+        l' = case r of
+              '\n' -> l + 1
+              _    -> l
+        c' = case r of
+              '\n' -> 1
+              _    -> c + 1
+       in if p r
+            then runP (runL ((r:) <$> lexWhile p) cont) rs c' l'
+            else runP (cont []) (r:rs) c l
 
 -- | lexWhile without the return value.
 lexWhile_ :: (Char -> Bool) -> Lex a ()
@@ -483,7 +525,7 @@ ignoreLinePragmasL = Lex $ \cont -> P $ \r x y loc s m ->
 setLineFilenameL :: String -> Lex a ()
 setLineFilenameL name = Lex $ \cont -> P $ \r x y loc s m ->
         runP (cont ()) r x y loc s (m {iParseFilename = name})
-        
+
 -- Comments
 
 pushComment :: Comment -> Lex a ()
