@@ -61,7 +61,7 @@ data PExp l
     | PreOp l (QOp l) (PExp l)              -- ^ pre-ops
 
 -- View patterns
-    | ViewPat l (PExp l) (PExp l)           -- ^ patterns only
+    | ViewPat l (PExp l) (Pat l)            -- ^ patterns only
 
 -- HaRP
     | SeqRP l [PExp l]                      -- ^ regular patterns only
@@ -306,6 +306,7 @@ data PType l
      | TyPromoted l (S.Promoted l)              -- ^ promoted data type
      | TySplice l (Splice l)                    -- ^ template haskell splice type
      | TyBang l (BangType l) (PType l)          -- ^ Strict type marked with \"@!@\" or type marked with UNPACK pragma.
+     | TyWildCard l (Maybe (Name l))            -- ^ Type wildcard
   deriving (Eq, Show, Functor)
 
 instance Annotated PType where
@@ -325,6 +326,7 @@ instance Annotated PType where
       TyPred l _                    -> l
       TySplice l _                  -> l
       TyBang  l _ _                 -> l
+      TyWildCard  l _               -> l
     amap f t' = case t' of
       TyForall l mtvs mcx t         -> TyForall (f l) mtvs mcx t
       TyFun   l t1 t2               -> TyFun (f l) t1 t2
@@ -341,31 +343,35 @@ instance Annotated PType where
       TyPred l asst                 -> TyPred (f l) asst
       TySplice l s                  -> TySplice (f l) s
       TyBang  l b t                 -> TyBang (f l) b t
+      TyWildCard l mn               -> TyWildCard (f l) mn
 
 data PAsst l
     = ClassA l (QName l) [PType l]
-    | VarA l (Name l)
+    | AppA l (Name l) [PType l]
     | InfixA l (PType l) (QName l) (PType l)
     | IParam l (IPName l) (PType l)
     | EqualP l (PType l)  (PType l)
     | ParenA l (PAsst l)
+    | WildCardA l (Maybe (Name l))
   deriving (Eq, Show, Functor)
 
 instance Annotated PAsst where
     ann asst = case asst of
         ClassA l _ _        -> l
-        VarA l _            -> l
+        AppA l _ _          -> l
         InfixA l _ _ _      -> l
         IParam l _ _        -> l
         EqualP l _ _        -> l
         ParenA l _          -> l
+        WildCardA l _       -> l
     amap f asst = case asst of
         ClassA l qn ts      -> ClassA (f l) qn ts
-        VarA l n            -> VarA (f l) n
+        AppA l t ts         -> AppA (f l) t ts
         InfixA l ta qn tb   -> InfixA (f l) ta qn tb
         IParam l ipn t      -> IParam (f l) ipn t
         EqualP l t1 t2      -> EqualP (f l) t1 t2
         ParenA l a          -> ParenA (f l) a
+        WildCardA l mn      -> WildCardA (f l) mn
 
 
 unit_tycon, fun_tycon, list_tycon, unboxed_singleton_tycon :: l -> PType l
