@@ -32,13 +32,16 @@ module Language.Haskell.Exts.ParseUtils (
     , checkPattern          -- PExp -> P Pat
     , checkExpr             -- PExp -> P Exp
     , checkType             -- PType -> P Type
+    , checkTyVar            -- Name  -> P PType
     , bangType              -- L -> BangType -> Type -> Type
     , checkKind             -- Kind -> P ()
     , checkValDef           -- SrcLoc -> PExp -> Maybe Type -> Rhs -> Binds -> P Decl
+    , checkExplicitPatSyn   --
     , checkClassBody        -- [ClassDecl] -> P [ClassDecl]
     , checkInstBody         -- [InstDecl] -> P [InstDecl]
     , checkUnQual           -- QName -> P Name
     , checkQualOrUnQual     -- QName -> P QName
+    , checkSingleDecl       -- [Decl] -> P Decl
     , checkRevDecls         -- [Decl] -> P [Decl]
     , checkRevClsDecls      -- [ClassDecl] -> P [ClassDecl]
     , checkRevInstDecls     -- [InstDecl] -> P [InstDecl]
@@ -49,6 +52,7 @@ module Language.Haskell.Exts.ParseUtils (
     , checkDefSigDef        -- Decl -> P Decl
     , getGConName           -- S.Exp -> P QName
     , mkTyForall            -- Maybe [TyVarBind] -> PContext -> PType -> PType
+    , mkRoleAnnotDecl       --
     -- HaRP
     , checkRPattern         -- PExp -> P RPat
     -- Hsx
@@ -70,7 +74,7 @@ module Language.Haskell.Exts.ParseUtils (
     ) where
 
 import Language.Haskell.Exts.Annotated.Syntax hiding ( Type(..), Asst(..), Exp(..), FieldUpdate(..), XAttr(..), Context(..) )
-import qualified Language.Haskell.Exts.Annotated.Syntax as S ( Type(..), Asst(..), Exp(..), FieldUpdate(..), XAttr(..), Context(..) )
+import qualified Language.Haskell.Exts.Annotated.Syntax as S ( Type(..), Asst(..), Exp(..), FieldUpdate(..), XAttr(..), Context(..), Role(..), PatternSynDirection(..))
 
 import Language.Haskell.Exts.ParseSyntax
 import Language.Haskell.Exts.ParseMonad
@@ -80,7 +84,7 @@ import Language.Haskell.Exts.Extension
 import Language.Haskell.Exts.ExtScheme
 
 import Prelude hiding (mod)
-import Data.List (intercalate)
+import Data.List (intercalate, intersperse)
 import Data.Maybe (fromJust, fromMaybe)
 import Control.Monad (when,unless)
 import Control.Applicative (Applicative (..), (<$>))
@@ -108,13 +112,28 @@ splitTyConApp t0 = do
 checkEnabled :: (Show e, Enabled e) => e  -> P ()
 checkEnabled e = do
     exts <- getExtensions
-    unless (isEnabled e exts) $ fail $ show e ++ " is not enabled"
+    unless (isEnabled e exts) $ fail errorMsg
+ where errorMsg = unwords
+          [ show e
+          , "language extension is not enabled."
+          , "Please add {-# LANGUAGE " ++ show e ++  " #-}"
+          , "pragma at the top of your module."
+          ]
 
 checkEnabledOneOf :: (Show e, Enabled e) => [e] -> P ()
 checkEnabledOneOf es = do
     exts <- getExtensions
     unless (any (`isEnabled` exts) es) $
-        fail $ (foldr1 (\x s -> x ++ " or " ++ s) . map show $ es) ++ " is not enabled"
+        fail errorMsg
+  where errorMsg = unwords
+          [ "At least one of"
+          , joinOr id
+          , "language extensions needs to be enabled."
+          , "Please add:"
+          , joinOr (\s -> "{-# LANGUAGE " ++ s ++ " #-}")
+          , "language pragma at the top of your module."
+          ]
+        joinOr f = concat . intersperse " or "  . map (f . show) $ es
 
 checkPatternGuards :: [Stmt L] -> P ()
 checkPatternGuards [Qualifier _ _] = return ()
@@ -194,9 +213,11 @@ checkAssertion t' = checkAssertion' id [] t'
                 return $ InfixA (fl l) a op b
             checkAssertion' fl ts (TyParen l t) =
                 checkAssertion' (const (fl l)) ts t
-            checkAssertion' fl [] (TyVar l t) = do -- Dict :: cxt => Dict cxt
+            checkAssertion' fl ts (TyVar l t) = do -- Dict :: cxt => Dict cxt
                 checkEnabled ConstraintKinds
-                return $ VarA (fl l) t
+                return $ AppA (fl l) t (reverse ts)
+            checkAssertion' _ _ (TyWildCard l wc) =
+                return $ WildCardA l wc
             checkAssertion' _ _ _ = fail "Illegal class assertion"
 
 getSymbol :: QName L -> Maybe String
@@ -240,7 +261,9 @@ checkAsst isSimple asst =
       ClassA l qn pts -> do
                 ts <- mapM (checkAsstParam isSimple) pts
                 return $ S.ClassA l qn ts
-      VarA l n        -> return $ S.VarA l n
+      AppA l n pts    -> do
+                ts <- mapM (checkAsstParam isSimple) pts
+                return $ S.AppA l n ts
       InfixA l a op b -> do
                 [a',b'] <- mapM (checkAsstParam isSimple) [a,b]
                 return $ S.InfixA l a' op b'
@@ -254,6 +277,9 @@ checkAsst isSimple asst =
       ParenA l a      -> do
                 a' <- checkAsst isSimple a
                 return $ S.ParenA l a'
+      WildCardA l a ->
+        if isSimple then fail "Malformed Context: WildCards not allowed in simple contexts"
+                    else return $ S.WildCardA l a
 
 checkAsstParam :: Bool -> PType L -> P (S.Type L)
 checkAsstParam isSimple t = do
@@ -262,6 +288,7 @@ checkAsstParam isSimple t = do
          then checkType t
          else case t of
                 TyVar l n     -> return $ S.TyVar l n
+                TyWildCard l mn ->  return $ S.TyWildCard l mn
                 TyParen l t1  -> do t1' <- checkAsstParam isSimple t1
                                     return $ S.TyParen l t1'
                 TyApp l pf pt | not isSimple    -> do
@@ -373,7 +400,9 @@ checkPattern :: PExp L -> P (Pat L)
 checkPattern e = checkPat e []
 
 checkPat :: PExp L -> [Pat L] -> P (Pat L)
-checkPat (Con l c) args = return (PApp l c args)
+checkPat (Con l c) args = do
+  let l' = foldl combSpanInfo l (map ann args)
+  return (PApp l' c args)
 checkPat (App _ f x) args = do
     x' <- checkPat x []
     checkPat f (x':args)
@@ -438,8 +467,7 @@ checkPat e' [] = case e' of
                   return (PIrrPat l p)
     ViewPat l e p  -> do
                   e1 <- checkExpr e
-                  p1 <- checkPat p []
-                  return (PViewPat l e1 p1)
+                  return (PViewPat l e1 p)
     RecConstr l c fs   -> do
                   fs' <- mapM checkPatField fs
                   return (PRec l c fs')
@@ -706,6 +734,9 @@ checkExpr e' = case e' of
     -- LamdaCase
     LCase l alts -> return $ S.LCase l alts
 
+    -- Hole
+    WildCard l     -> return $ S.ExprHole l
+
     _             -> fail $ "Parse error in expression: " ++ prettyPrint e'
 
 checkAttr :: ParseXAttr L -> P (S.XAttr L)
@@ -801,9 +832,13 @@ isFunLhs (InfixApp _ l (QVarOp loc (UnQual _ op)) r) es
         exts <- getExtensions
         if BangPatterns `elem` exts
          then let (b,bs) = splitBang r []
-               in isFunLhs l (BangPat loc b : bs ++ es)
+                  loc' = combSpanInfo loc (ann b)
+               in isFunLhs l (BangPat loc' b : bs ++ es)
          else return $ Just (op, l:r:es, False, []) -- It's actually a definition of the operator !
-    | otherwise = return $ Just (op, l:r:es, False, [])
+    | otherwise =
+        let infos = srcInfoPoints loc
+            op'   = amap (\s -> s { srcInfoPoints = infos }) op
+        in (return $ Just (op', l:r:es, False, []))
 isFunLhs (App _ (Var _ (UnQual _ f)) e) es = return $ Just (f, e:es, True, [])
 isFunLhs (App _ f e) es = isFunLhs f (e:es)
 isFunLhs (Var _ (UnQual _ f)) es@(_:_) = return $ Just (f, es, True, [])
@@ -822,6 +857,19 @@ checkSigVar :: PExp L -> P (Name L)
 checkSigVar (Var _ (UnQual l n)) = return $ fmap (const l) n
 checkSigVar e = fail $ "Left-hand side of type signature is not a variable: " ++ prettyPrint e
 
+checkExplicitPatSyn :: S -> S -> ([Decl L], [S]) -> S -> P (PatternSynDirection L)
+checkExplicitPatSyn whereLoc openLoc (decls, semis) closeLoc =
+  let l = whereLoc <^^> closeLoc  <** ([whereLoc, openLoc] ++ semis ++ [closeLoc])
+  in  S.ExplicitBidirectional l  <$> mapM checkDecls decls
+  where
+    checkDecls :: Decl L -> P (Decl L)
+    checkDecls p@(PatBind _ pat _ _) =
+      case pat of
+        PApp _ _ _        -> return p
+        PInfixApp _ _ _ _ ->  return p
+        _ -> fail "Illegal pattern binding in PatternSynonym"
+    checkDecls _                 = fail "pattern synonym 'where' clause must contain a PatBind"
+
 -----------------------------------------------------------------------------
 -- In a class or instance body, a pattern binding must be of a variable.
 
@@ -899,6 +947,17 @@ updateQNameLoc l (UnQual _ n)  = UnQual l n
 updateQNameLoc l (Special _ s) = Special l s
 
 -----------------------------------------------------------------------------
+-- For standalone top level Decl parser, check that we actually only
+-- parsed one Decl. This is needed since we parse matches of the same
+-- FunBind as multiple separate declarations, and merge them after.
+-- This should be called *after* checkRevDecls.
+
+checkSingleDecl :: [Decl L] -> P (Decl L)
+checkSingleDecl [d] = return d
+checkSingleDecl ds =
+    fail $ "Expected a single declaration, found " ++ show (length ds)
+
+
 -- Reverse a list of declarations, merging adjacent FunBinds of the
 -- same name and checking that their arities match.
 
@@ -911,11 +970,12 @@ checkRevDecls = mergeFunBinds []
         where
         arity = length ps
         mergeMatches ms' (FunBind _ ms@(Match loc name' ps' _ _:_):ds) l
-            | name' =~= name =
-            if length ps' /= arity
-            then fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
+            | name' =~= name = do
+            ignoreArity <- getIgnoreFunctionArity
+            if length ps' == arity || ignoreArity
+              then mergeMatches (ms++ms') ds (loc <++> l)
+              else fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
                     `atSrcLoc` fromSrcInfo loc
-            else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeFunBinds (FunBind l ms':revDs) ds
     mergeFunBinds revDs (FunBind l' ims1@(InfixMatch _ _ name _ _ _:_):ds1) =
         mergeInfix ims1 ds1 l'
@@ -935,11 +995,12 @@ checkRevClsDecls = mergeClsFunBinds []
         where
         arity = length ps
         mergeMatches ms' (ClsDecl _ (FunBind _ ms@(Match loc name' ps' _ _:_)):ds) l
-            | name' =~= name =
-            if length ps' /= arity
-            then fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
+            | name' =~= name = do
+            ignoreArity <- getIgnoreFunctionArity
+            if length ps' == arity || ignoreArity
+              then mergeMatches (ms++ms') ds (loc <++> l)
+              else fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
                     `atSrcLoc` fromSrcInfo loc
-            else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeClsFunBinds (ClsDecl l (FunBind l ms'):revDs) ds
     mergeClsFunBinds revDs (ClsDecl l' (FunBind _ ims1@(InfixMatch _ _ name _ _ _:_)):ds1) =
         mergeInfix ims1 ds1 l'
@@ -960,11 +1021,12 @@ checkRevInstDecls = mergeInstFunBinds []
         where
         arity = length ps
         mergeMatches ms' (InsDecl _ (FunBind _ ms@(Match loc name' ps' _ _:_)):ds) l
-            | name' =~= name =
-            if length ps' /= arity
-            then fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
+            | name' =~= name = do
+            ignoreArity <- getIgnoreFunctionArity
+            if length ps' == arity || ignoreArity
+              then mergeMatches (ms++ms') ds (loc <++> l)
+              else fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
                     `atSrcLoc` fromSrcInfo loc
-            else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeInstFunBinds (InsDecl l (FunBind l ms'):revDs) ds
     mergeInstFunBinds revDs (InsDecl l' (FunBind _ ims1@(InfixMatch _ _ name _ _ _:_)):ds1) =
         mergeInfix ims1 ds1 l'
@@ -1030,16 +1092,17 @@ checkT t simple = case t of
     TyInfix l at op bt -> checkAndWarnTypeOperators op >> check2Types at bt (flip (S.TyInfix l) op)
     TyKind  l pt k    -> check1Type pt (flip (S.TyKind l) k)
 
-     -- TyPred can be a valid type if ConstraintKinds is enabled, unless it is an implicit parameter, which is not a valid type 
+     -- TyPred can be a valid type if ConstraintKinds is enabled, unless it is an implicit parameter, which is not a valid type
     TyPred _ (ClassA l className cvars) -> mapM checkType cvars >>= \vars -> return (foldl1 (S.TyApp l) (S.TyCon l className:vars))
-    TyPred _ (InfixA l t0 op t1)        -> S.TyInfix l <$> checkType t0 <*> pure op <*> checkType t1 
-    TyPred _ (EqualP l t0    t1)        -> S.TyEquals l <$> checkType t0 <*> checkType t1 where 
+    TyPred _ (InfixA l t0 op t1)        -> S.TyInfix l <$> checkType t0 <*> pure op <*> checkType t1
+    TyPred _ (EqualP l t0    t1)        -> S.TyEquals l <$> checkType t0 <*> checkType t1 where
 
     TyPromoted l p -> return $ S.TyPromoted l p -- ??
     TySplice l s        -> do
                               checkEnabled TemplateHaskell
                               return $ S.TySplice l s
     TyBang l b t' -> check1Type t' (S.TyBang l b)
+    TyWildCard l mn -> return $ S.TyWildCard l mn
     _   -> fail $ "Parse error in type: " ++ prettyPrint t
 
 check1Type :: PType L -> (S.Type L -> S.Type L) -> P (S.Type L)
@@ -1051,6 +1114,21 @@ check2Types at bt f = checkT at True >>= \a -> checkT bt True >>= \b -> return (
 checkTypes :: [PType L] -> P [S.Type L]
 checkTypes = mapM (flip checkT True)
 
+checkTyVar ::  Name L -> P (PType L)
+checkTyVar n = do
+  e <- getExtensions
+  return $
+    case n of
+      Ident il ('_':ident) | NamedWildCards `elem` e ->
+        TyWildCard il (Just (Ident (reduceSrcSpanInfo il) ident))
+      _ ->
+        TyVar (ann n) n
+  where
+    -- Reduces the length of the SrcSpanInfo by 1 so that it just covers the identifier.
+    reduceSrcSpanInfo spaninfo =
+      let ss = srcInfoSpan spaninfo
+          ss' = ss { srcSpanStartColumn = srcSpanStartColumn ss + 1 }
+      in  spaninfo { srcInfoSpan = ss' }
 ---------------------------------------
 -- Check kinds
 
@@ -1104,3 +1182,26 @@ mkTyForall l mtvs ctxt ty =
     case (ctxt, ty) of
         (Nothing, TyForall _ Nothing ctxt2 ty2) -> TyForall l mtvs ctxt2 ty2
         _                                       -> TyForall l mtvs ctxt ty
+
+-- Make a role annotation
+
+mkRoleAnnotDecl ::  S -> S -> QName L -> [(Maybe String, L)] -> P (Decl L)
+mkRoleAnnotDecl l1 l2 tycon roles
+  = do roles' <- mapM parse_role roles
+       return (RoleAnnotDecl loc' tycon roles')
+  where
+    loc' =
+      case roles of
+        [] -> (l1 <^^> l2 <++> ann tycon) <** [l1, l2]
+        _  -> (l1 <^^> l2 <++> ann tycon <++> foldl1 (<++>) (map snd roles)) <** [l1, l2]
+    possible_roles = [ ("phantom", S.Phantom)
+                     , ("representational", S.Representational)
+                     , ("nominal", S.Nominal)]
+
+    parse_role (Nothing, loc_role) = return $ S.RoleWildcard loc_role
+    parse_role (Just role, loc_role)
+      = case lookup role possible_roles of
+          Just found_role -> return $ found_role loc_role
+          Nothing         ->
+            fail ("Illegal role name " ++ role)
+
