@@ -20,7 +20,10 @@ module Language.Haskell.Exts.Pretty (
                 -- * Pretty-printing styles (from "Text.PrettyPrint.HughesPJ")
                 P.Style(..), P.style, P.Mode(..),
                 -- * Haskell formatting modes
-                PPHsMode(..), Indent, PPLayout(..), defaultMode) where
+                PPHsMode(..), Indent, PPLayout(..), defaultMode
+                -- * Primitive Printers
+                , prettyPrim, prettyPrimWithMode
+                ) where
 
 import Language.Haskell.Exts.Syntax
 import qualified Language.Haskell.Exts.Annotated.Syntax as A
@@ -32,6 +35,7 @@ import Language.Haskell.Exts.SrcLoc hiding (loc)
 import Prelude hiding (exp)
 import qualified Text.PrettyPrint as P
 import Data.List (intersperse)
+import Data.Maybe (isJust)
 import Control.Applicative (Applicative(..))
 import qualified Control.Monad as M (ap)
 
@@ -221,8 +225,8 @@ rbrace = return  P.rbrace
 aM <> bM = do{a<-aM;b<-bM;return (a P.<> b)}
 aM <+> bM = do{a<-aM;b<-bM;return (a P.<+> b)}
 aM $$ bM = do{a<-aM;b<-bM;return (a P.$$ b)}
--- ($+$) :: Doc -> Doc -> Doc
--- aM $+$ bM = do{a<-aM;b<-bM;return (a P.$+$ b)}
+($+$) :: Doc -> Doc -> Doc
+aM $+$ bM = do{a<-aM;b<-bM;return (a P.$+$ b)}
 
 hcat,hsep,vcat,fsep :: [Doc] -> Doc
 hcat dl = sequence dl >>= return . P.hcat
@@ -281,6 +285,15 @@ prettyPrint = prettyPrintWithMode defaultMode
 --               -> a -> Doc -> a
 -- fullRender = fullRenderWithMode defaultMode
 
+-- | pretty-print with the default style and 'defaultMode'.
+prettyPrim :: Pretty a => a -> P.Doc
+prettyPrim = prettyPrimWithMode defaultMode
+
+-- | pretty-print with the default style and a given mode.
+prettyPrimWithMode :: Pretty a => PPHsMode -> a -> P.Doc
+prettyPrimWithMode pphs doc = unDocM (pretty doc) pphs
+
+
 -------------------------  Pretty-Print a Module --------------------
 instance Pretty Module where
         pretty (Module pos m os mbWarn mbExports imp decls) =
@@ -288,7 +301,11 @@ instance Pretty Module where
                 myVcat $ map pretty os ++
                     (if m == ModuleName "" then id
                      else \x -> [topLevel (ppModuleHeader m mbWarn mbExports) x])
-                    (map pretty imp ++ map pretty decls)
+                    (map pretty imp ++
+                      ppDecls (m /= ModuleName "" ||
+                               not (null imp) ||
+                               not (null os))
+                              decls)
 
 --------------------------  Module Header ------------------------------
 ppModuleHeader :: ModuleName -> Maybe WarningText -> Maybe [ExportSpec] -> Doc
@@ -306,11 +323,14 @@ ppWarnTxt (WarnText s) = mySep [text "{-# WARNING",    text (show s), text "#-}"
 instance Pretty ModuleName where
         pretty (ModuleName modName) = text modName
 
+instance Pretty Namespace where
+        pretty NoNamespace      = empty
+        pretty TypeNamespace    = text "type"
+        pretty PatternNamespace = text "pattern"
+
 instance Pretty ExportSpec where
-        pretty (EVar t name)              =
-                (case t of { NoNamespace -> empty; TypeNamespace -> text "type" })
-                  <+> pretty name
-        pretty (EAbs name)                = pretty name
+        pretty (EVar name)                = pretty name
+        pretty (EAbs ns name)             = pretty ns <+> pretty name
         pretty (EThingAll name)           = pretty name <> text "(..)"
         pretty (EThingWith name nameList) =
                 pretty name <> (parenList . map pretty $ nameList)
@@ -333,10 +353,8 @@ instance Pretty ImportDecl where
                     where specs = parenList . map pretty $ specList
 
 instance Pretty ImportSpec where
-        pretty (IVar t name)              =
-                (case t of { NoNamespace -> empty; TypeNamespace -> text "type" })
-                  <+> pretty name
-        pretty (IAbs name)                = pretty name
+        pretty (IVar name  )              = pretty name
+        pretty (IAbs ns name)             = pretty ns <+> pretty name
         pretty (IThingAll name)           = pretty name <> text "(..)"
         pretty (IThingWith name nameList) =
                 pretty name <> (parenList . map pretty $ nameList)
@@ -345,16 +363,33 @@ instance Pretty TypeEqn where
         pretty (TypeEqn pat eqn) = mySep [pretty pat, equals, pretty eqn]
 
 -------------------------  Declarations ------------------------------
+class Pretty a => PrettyDeclLike a where
+  wantsBlankline :: a -> Bool
+
+instance PrettyDeclLike Decl where
+  wantsBlankline (FunBind _) = False
+  wantsBlankline (PatBind {}) = False
+  wantsBlankline _ = True
+
+condBlankline :: PrettyDeclLike a => a -> Doc
+condBlankline d = (if wantsBlankline d then blankline else id) $ pretty d
+
+ppDecls :: PrettyDeclLike a => Bool -> [a] -> [Doc]
+ppDecls True  ds     = map condBlankline ds
+ppDecls False (d:ds) = pretty d : map condBlankline ds
+ppDecls _ _ = []
+--ppDecls = map condBlankline
+
 instance Pretty Decl where
         pretty (TypeDecl loc name nameList htype) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ( [text "type", pretty name]
                         ++ map pretty nameList
                         ++ [equals, pretty htype])
 
         pretty (DataDecl loc don context name nameList constrList derives) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ( [pretty don, ppContext context, pretty name]
                         ++ map pretty nameList)
@@ -363,7 +398,7 @@ instance Pretty Decl where
                         $$$ ppDeriving derives)
 
         pretty (GDataDecl loc don context name nameList optkind gadtList derives) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ( [pretty don, ppContext context, pretty name]
                         ++ map pretty nameList ++ ppOptKind optkind ++ [text "where"])
@@ -371,32 +406,32 @@ instance Pretty Decl where
                         $$$ ppIndent letIndent [ppDeriving derives]
 
         pretty (TypeFamDecl loc name nameList optkind) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ([text "type", text "family", pretty name]
                         ++ map pretty nameList
                         ++ ppOptKind optkind)
 
         pretty (ClosedTypeFamDecl loc name nameList optkind eqns) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ([text "type", text "family", pretty name]
                         ++ map pretty nameList
                         ++ ppOptKind optkind ++ [text "where"]) $$$ ppBody classIndent (map pretty eqns)
 
         pretty (DataFamDecl loc context name nameList optkind) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ( [text "data", text "family", ppContext context, pretty name]
                         ++ map pretty nameList ++ ppOptKind optkind)
 
         pretty (TypeInsDecl loc ntype htype) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep [text "type", text "instance", pretty ntype, equals, pretty htype]
 
         pretty (DataInsDecl loc don ntype constrList derives) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep [pretty don, text "instance", pretty ntype]
                         <+> (myVcat (zipWith (<+>) (equals : repeat (char '|'))
@@ -404,7 +439,7 @@ instance Pretty Decl where
                               $$$ ppDeriving derives)
 
         pretty (GDataInsDecl loc don ntype optkind gadtList derives) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep ( [pretty don, text "instance", pretty ntype]
                         ++ ppOptKind optkind ++ [text "where"])
@@ -414,56 +449,70 @@ instance Pretty Decl where
         --m{spacing=False}
         -- special case for empty class declaration
         pretty (ClassDecl pos context name nameList fundeps []) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep ( [text "class", ppContext context, pretty name]
                         ++ map pretty nameList ++ [ppFunDeps fundeps])
         pretty (ClassDecl pos context name nameList fundeps declList) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep ( [text "class", ppContext context, pretty name]
                         ++ map pretty nameList ++ [ppFunDeps fundeps, text "where"])
-                $$$ ppBody classIndent (map pretty declList)
+                $$$ ppBody classIndent (ppDecls False declList)
 
         -- m{spacing=False}
         -- special case for empty instance declaration
         pretty (InstDecl pos overlap tvs context name args []) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 let olp = case overlap of { Nothing -> empty; Just o -> space <> pretty o }
                 in mySep ( [text "instance" <> olp, ppForall (Just tvs)
                            , ppContext context, pretty name] ++ map ppAType args)
         pretty (InstDecl pos overlap tvs context name args declList) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 let olp = case overlap of { Nothing -> empty; Just o -> space <> pretty o }
                 in mySep ( [text "instance" <> olp,  ppForall (Just tvs)
                            , ppContext context, pretty name]
                           ++ map ppAType args ++ [text "where"])
-                $$$ ppBody classIndent (map pretty declList)
+                $$$ ppBody classIndent (ppDecls False declList)
 
         pretty (DerivDecl pos overlap tvs context name args) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 let olp = case overlap of { Nothing -> empty; Just o -> space <> pretty o }
                 in mySep ( [text "deriving", text "instance" <> olp, ppForall (Just tvs)
                            , ppContext context, pretty name] ++ map ppAType args)
         pretty (DefaultDecl pos htypes) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 text "default" <+> parenList (map pretty htypes)
 
         pretty (SpliceDecl pos splice) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 pretty splice
 
         pretty (TypeSig pos nameList qualType) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep ((punctuate comma . map pretty $ nameList)
                       ++ [text "::", pretty qualType])
 
+        --  Req can be ommitted if it is empty
+        --  We must print prov if req is nonempty
+        pretty (PatSynSig pos n mtvs prov req t) =
+                let contexts = case (prov, req) of
+                                    ([], []) -> []
+                                    (ps, []) -> [ppContext ps]
+                                    ([], rs) -> [text "()" <+> text "=>", ppContext rs]
+                                    (ps, rs) -> [ppContext ps, ppContext rs]
+                 in
+                  markLine pos $
+                  mySep ( [text "pattern", pretty n, text "::", ppForall mtvs] ++
+                          contexts ++ [pretty t] )
+
+
         pretty (FunBind matches) = do
                 e <- fmap layout getPPEnv
                 case e of PPOffsideRule -> foldr ($$$) empty (map pretty matches)
@@ -474,79 +523,99 @@ instance Pretty Decl where
                 myFsep [pretty pat, pretty rhs] $$$ ppWhere whereBinds
 
         pretty (InfixDecl pos assoc prec opList) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep ([pretty assoc, int prec]
                        ++ (punctuate comma . map pretty $ opList))
 
+        pretty (PatSyn pos pat rhs dir) =
+                let sep = case dir of
+                            ImplicitBidirectional   -> "="
+                            ExplicitBidirectional _ -> "<-"
+                            Unidirectional          -> "<-"
+                in
+                markLine pos $
+                 (mySep ([text "pattern", pretty pat, text sep, pretty rhs])) $$$
+                    (case dir of
+                      ExplicitBidirectional ds ->
+                        nest 2 (text "where" $$$ ppBody whereIndent (ppDecls False ds))
+                      _ -> empty)
+
         pretty (ForImp pos cconv saf str name typ) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep [text "foreign import", pretty cconv, pretty saf,
                        text (show str), pretty name, text "::", pretty typ]
 
         pretty (ForExp pos cconv str name typ) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep [text "foreign export", pretty cconv,
                        text (show str), pretty name, text "::", pretty typ]
 
         pretty (RulePragmaDecl pos rules) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 myVcat $ text "{-# RULES" : map pretty rules ++ [text " #-}"]
 
         pretty (DeprPragmaDecl pos deprs) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 myVcat $ text "{-# DEPRECATED" : map ppWarnDepr deprs ++ [text " #-}"]
 
         pretty (WarnPragmaDecl pos deprs) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 myVcat $ text "{-# WARNING" : map ppWarnDepr deprs ++ [text " #-}"]
 
         pretty (InlineSig pos inl activ name) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep [text (if inl then "{-# INLINE" else "{-# NOINLINE"), pretty activ, pretty name, text "#-}"]
 
         pretty (InlineConlikeSig pos activ name) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep [text "{-# INLINE CONLIKE", pretty activ, pretty name, text "#-}"]
 
         pretty (SpecSig pos activ name types) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep $ [text "{-# SPECIALISE", pretty activ, pretty name, text "::"]
                     ++ punctuate comma (map pretty types) ++ [text "#-}"]
 
         pretty (SpecInlineSig pos inl activ name types) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep $ [text "{-# SPECIALISE", text (if inl then "INLINE" else "NOINLINE"),
                         pretty activ, pretty name, text "::"]
                         ++ (punctuate comma $ map pretty types) ++ [text "#-}"]
 
         pretty (InstSig pos tvs context name args) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep $ [text "{-# SPECIALISE", text "instance"
                          , ppForall (Just tvs), ppContext context, pretty name]
                          ++ map ppAType args ++ [text "#-}"]
 
         pretty (AnnPragma pos ann) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 mySep [text "{-# ANN", pretty ann, text "#-}"]
 
         pretty (MinimalPragma pos b) =
-                blankline $
+                -- blankline $
                 markLine pos $
                 let bs = case b of { Just b' -> pretty b'; _ -> empty }
                 in myFsep [text "{-# MINIMAL", bs, text "#-}"]
 
+        pretty (RoleAnnotDecl pos qn rs) =
+                -- blankline $
+                markLine pos $
+                mySep ( [text "type", text "role", pretty qn]
+                        ++ map pretty rs )
+
+
 instance Pretty Annotation where
         pretty (Ann n e) = myFsep [pretty n, pretty e]
         pretty (TypeAnn n e) = myFsep [text "type", pretty n, pretty e]
@@ -558,6 +627,12 @@ instance Pretty BooleanFormula where
         pretty (OrFormula bs)   = myFsep $ punctuate (text " |") $ map pretty bs
         pretty (ParenFormula b) = parens $ pretty b
 
+instance Pretty Role where
+        pretty RoleWildcard     = char '_'
+        pretty Nominal          = text "nominal"
+        pretty Representational = text "representational"
+        pretty Phantom          = text "phantom"
+
 instance Pretty DataOrNew where
         pretty DataType = text "data"
         pretty NewType  = text "newtype"
@@ -580,14 +655,19 @@ instance Pretty Match where
                                 else parens (myFsep hd) : map (prettyPrec 3) ps'
                         _ -> pretty f : map (prettyPrec 3) ps
 
-ppWhere :: Binds -> Doc
-ppWhere (BDecls []) = empty
-ppWhere (BDecls l)  = nest 2 (text "where" $$$ ppBody whereIndent (map pretty l))
-ppWhere (IPBinds b) = nest 2 (text "where" $$$ ppBody whereIndent (map pretty b))
+ppWhere :: Maybe Binds -> Doc
+ppWhere Nothing            = empty
+ppWhere (Just (BDecls l))  = nest 2 (text "where" $$$ ppBody whereIndent (ppDecls False l))
+ppWhere (Just (IPBinds b)) = nest 2 (text "where" $$$ ppBody whereIndent (ppDecls False b))
 
 ppSig :: Type -> Doc
 ppSig t = text "::" <+> pretty t
 
+instance PrettyDeclLike ClassDecl where
+    wantsBlankline (ClsDecl d) = wantsBlankline d
+    wantsBlankline (ClsDefSig {}) = True
+    wantsBlankline _ = False
+
 instance Pretty ClassDecl where
     pretty (ClsDecl decl) = pretty decl
 
@@ -606,7 +686,7 @@ instance Pretty ClassDecl where
                 mySep [text "type", pretty ntype, equals, pretty htype]
 
     pretty (ClsDefSig loc name typ) =
-                blankline $
+                -- blankline $
                 markLine loc $
                 mySep [
                     text "default",
@@ -614,6 +694,11 @@ instance Pretty ClassDecl where
                     text "::",
                     pretty typ]
 
+
+instance PrettyDeclLike InstDecl where
+    wantsBlankline (InsDecl d) = wantsBlankline d
+    wantsBlankline _ = False
+
 instance Pretty InstDecl where
         pretty (InsDecl decl) = pretty decl
 
@@ -647,13 +732,14 @@ instance Pretty Safety where
         pretty PlayInterruptible = text "interruptible"
 
 instance Pretty CallConv where
-        pretty StdCall   = text "stdcall"
-        pretty CCall     = text "ccall"
-        pretty CPlusPlus = text "cplusplus"
-        pretty DotNet    = text "dotnet"
-        pretty Jvm       = text "jvm"
-        pretty Js        = text "js"
-        pretty CApi      = text "capi"
+        pretty StdCall    = text "stdcall"
+        pretty CCall      = text "ccall"
+        pretty CPlusPlus  = text "cplusplus"
+        pretty DotNet     = text "dotnet"
+        pretty Jvm        = text "jvm"
+        pretty Js         = text "js"
+        pretty JavaScript = text "javascript"
+        pretty CApi       = text "capi"
 
 ------------------------- Pragmas ---------------------------------------
 ppWarnDepr :: ([Name], String) -> Doc
@@ -683,13 +769,21 @@ instance Pretty RuleVar where
     pretty (RuleVar n) = pretty n
     pretty (TypedRuleVar n t) = parens $ mySep [pretty n, text "::", pretty t]
 
+-- Spaces are stripped from the pragma text but other whitespace
+-- is not.
+ppOptionsPragma :: Doc -> String -> Doc
+ppOptionsPragma opt s =
+  case s of
+    ('\n':_) -> opt <> text s <> text "#-}"
+    _ ->  myFsep [opt, text s <> text "#-}"]
+
 instance Pretty ModulePragma where
     pretty (LanguagePragma _ ns) =
         myFsep $ text "{-# LANGUAGE" : punctuate (char ',') (map pretty ns) ++ [text "#-}"]
     pretty (OptionsPragma _ (Just tool) s) =
-        myFsep [text "{-# OPTIONS_" <> pretty tool <> text s <> text "#-}"]
+        ppOptionsPragma (text "{-# OPTIONS_" <> pretty tool) s
     pretty (OptionsPragma _ _ s) =
-        myFsep [text "{-# OPTIONS" <> text s <> text "#-}"]
+        ppOptionsPragma (text "{-# OPTIONS") s
     pretty (AnnModulePragma _ ann) =
         myFsep [text "{-# ANN", pretty ann, text "#-}"]
 
@@ -720,7 +814,7 @@ instance Pretty ConDecl where
                 myFsep [prettyPrec prec_btype l, ppName name,
                         prettyPrec prec_btype r] -}
         pretty (ConDecl name typeList) =
-                mySep $ ppName name : map (prettyPrec prec_atype) typeList
+                mySep $ pretty name : map (prettyPrec prec_atype) typeList
         pretty (InfixConDecl l name r) =
                 myFsep [prettyPrec prec_btype l, ppNameInfix name,
                          prettyPrec prec_btype r]
@@ -780,7 +874,9 @@ instance Pretty Type where
         prettyPrec _ (TyPromoted p) = pretty p
         prettyPrec p (TyEquals a b) = parensIf (p > 0) (myFsep [pretty a, text "~", pretty b])
         prettyPrec _ (TySplice s) = pretty s
-        prettyPrec _ (TyBang b t) = pretty b <> pretty t
+        prettyPrec _ (TyBang b t) = pretty b <> prettyPrec prec_atype t
+        prettyPrec _ (TyWildCard mn) = char '_' <> maybePP pretty mn
+
 
 instance Pretty Promoted where
   pretty p =
@@ -811,12 +907,11 @@ ppForall (Just vs) =    myFsep (text "forall" : map pretty vs ++ [char '.'])
 
 instance Pretty Kind where
         prettyPrec _ KindStar      = text "*"
-        prettyPrec _ KindBang      = text "!"
         prettyPrec n (KindFn a b)  = parensIf (n > 0) $ myFsep [prettyPrec 1 a, text "->", pretty b]
         prettyPrec _ (KindParen k) = parens $ pretty k
         prettyPrec _ (KindVar n)   = pretty n
         prettyPrec _ (KindTuple t) = parenList . map pretty $ t
-        prettyPrec _ (KindList l)  = bracketList . punctuate comma . map pretty $ l
+        prettyPrec _ (KindList l)  = brackets .  pretty $ l
         prettyPrec n (KindApp a b) =
           parensIf (n > 3) $ myFsep [prettyPrec 3 a, prettyPrec 4 b]
 
@@ -988,6 +1083,7 @@ instance Pretty Exp where
         -- LamdaCase
         prettyPrec p (LCase altList) = parensIf (p > 1) $
                 text "\\case" $$$ ppBody caseIndent (map pretty altList)
+        prettyPrec _ ExprHole       = char '_'
 
 
 instance Pretty XAttr where
@@ -998,8 +1094,8 @@ instance Pretty XName where
         pretty (XName n) = text n
         pretty (XDomName d n) = text d <> char ':' <> text n
 
-ppLetExp :: (Pretty a, Pretty b) => [a] -> b -> Doc
-ppLetExp l b = myFsep [text "let" <+> ppBody letIndent (map pretty l),
+ppLetExp :: (PrettyDeclLike a, Pretty b) => [a] -> b -> Doc
+ppLetExp l b = myFsep [text "let" <+> ppBody letIndent (ppDecls False l),
                         text "in", pretty b]
 
 -- ppWith :: Pretty a => [a] -> Doc
@@ -1014,7 +1110,7 @@ instance Pretty Bracket where
         pretty (PatBracket p) = ppBracket "[p|" p
         pretty (TypeBracket t) = ppBracket "[t|" t
         pretty (DeclBracket d) =
-                myFsep $ text "[d|" : map pretty d ++ [text "|]"]
+                myFsep $ text "[d|" : ppDecls True d ++ [text "|]"]
 
 ppBracket :: Pretty a => String -> a -> Doc
 ppBracket o x = myFsep [text o, pretty x, text "|]"]
@@ -1095,9 +1191,11 @@ instance Pretty RPat where
         pretty (RPEither r1 r2) = parens . myFsep $
                 [pretty r1, char '|', pretty r2]
         pretty (RPSeq rs) =
-                myFsep $ text "(/" : map pretty rs ++ [text "/)"]
+                myFsep $ text "(|" : (punctuate comma . map pretty $ rs)
+                           ++ [text "|)"]
         pretty (RPGuard r gs) =
-                myFsep $ text "(|" : pretty r : char '|' : map pretty gs ++ [text "|)"]
+                myFsep $ text "(|" : pretty r : char '|' :
+                           (punctuate comma . map pretty $ gs) ++ [text "|)"]
         -- special case that would otherwise be buggy
         pretty (RPCAs n (RPPat (PIrrPat p))) =
                 myFsep [pretty n <> text "@:", char '~' <> pretty p]
@@ -1198,6 +1296,9 @@ instance Pretty IPName where
         pretty (IPDup s) = char '?' <> text s
         pretty (IPLin s) = char '%' <> text s
 
+instance PrettyDeclLike IPBind where
+  wantsBlankline _ = False
+
 instance Pretty IPBind where
         pretty (IPBind _loc ipname exp) =
                 myFsep [pretty ipname, equals, pretty exp]
@@ -1244,12 +1345,13 @@ ppContext context = mySep [parenList (map pretty context), text "=>"]
 
 -- hacked for multi-parameter type classes
 instance Pretty Asst where
-        pretty (ClassA a ts)   = myFsep $ ppQName a : map ppAType ts
-        pretty (VarA n)        = pretty n
+        pretty (ClassA a ts)   = myFsep $ pretty a : map ppAType ts
+        pretty (AppA n ns)     = myFsep $ pretty n : map pretty ns
         pretty (InfixA a op b) = myFsep [pretty a, ppQNameInfix op, pretty b]
         pretty (IParam i t)    = myFsep [pretty i, text "::", pretty t]
         pretty (EqualP t1 t2)  = myFsep [pretty t1, text "~", pretty t2]
         pretty (ParenA a)      = parens (pretty a)
+        pretty (WildCardA mn)  = char  '_' <> maybePP pretty mn
 
 -- Pretty print a source location, useful for printing out error messages
 instance Pretty SrcLoc where
@@ -1292,7 +1394,11 @@ instance SrcInfo pos => Pretty (A.Module pos) where
                     (case mbHead of
                         Nothing -> id
                         Just h  -> \x -> [topLevel (pretty h) x])
-                    (map pretty imp ++ map pretty decls)
+                    (map pretty imp ++
+                         ppDecls (isJust mbHead ||
+                                  not (null imp) ||
+                                  not (null os))
+                           decls)
         pretty (A.XmlPage pos _mn os n attrs mattr cs) =
                 markLine pos $
                 myVcat $ map pretty os ++
@@ -1306,7 +1412,8 @@ instance SrcInfo pos => Pretty (A.Module pos) where
                     (case mbHead of
                         Nothing -> id
                         Just h  -> \x -> [topLevel (pretty h) x])
-                    (map pretty imp ++ map pretty decls ++
+                    (map pretty imp ++
+                      ppDecls (isJust mbHead || not (null imp) || not (null os)) decls ++
                         [let ax = maybe [] (return . pretty) mattr
                           in hcat $
                              (myFsep $ (char '<' <> pretty n): map pretty attrs ++ ax ++ [char '>']):
@@ -1350,6 +1457,9 @@ instance Pretty (A.ImportSpec l) where
 instance SrcInfo pos => Pretty (A.Decl pos) where
         pretty = pretty . sDecl
 
+instance SrcInfo pos => PrettyDeclLike (A.Decl pos) where
+        wantsBlankline = wantsBlankline . sDecl
+
 instance Pretty (A.DeclHead l) where
     pretty (A.DHead _ n)           = pretty n
     pretty (A.DHInfix _ tva n)     = mySep [pretty tva, pretty n]
@@ -1540,6 +1650,9 @@ instance Pretty (A.IPName l) where
 instance SrcInfo loc => Pretty (A.IPBind loc) where
         pretty = pretty . sIPBind
 
+instance SrcInfo loc => PrettyDeclLike (A.IPBind loc) where
+        wantsBlankline = wantsBlankline . sIPBind
+
 instance Pretty (A.CName l) where
         pretty = pretty . sCName
 
@@ -1563,7 +1676,7 @@ parenList = parens . myFsepSimple . punctuate comma
 hashParenList :: [Doc] -> Doc
 hashParenList = hashParens . myFsepSimple . punctuate comma
   where hashParens = parens . hashes
-        hashes doc = char '#' <> doc <> char '#'
+        hashes doc = char '#' <+> doc <+> char '#'
 
 braceList :: [Doc] -> Doc
 braceList = braces . myFsepSimple . punctuate comma
@@ -1589,7 +1702,7 @@ prettyBlock = braces . (space <>) . vcat . punctuate semi
 
 blankline :: Doc -> Doc
 blankline dl = do{e<-getPPEnv;if spacing e && layout e /= PPNoLayout
-                              then space $$ dl else dl}
+                              then text "" $+$ dl else dl}
 topLevel :: Doc -> [Doc] -> Doc
 topLevel header dl = do
          e <- fmap layout getPPEnv
@@ -1769,9 +1882,11 @@ instance SrcInfo loc => Pretty (P.PExp loc) where
         pretty (P.PreOp _ op e)  = pretty op <+> pretty e
         pretty (P.ViewPat _ e p) =
                 myFsep [pretty e, text "->", pretty p]
-        pretty (P.SeqRP _ rs) = myFsep $ text "(/" : map pretty rs ++ [text "/)"]
+        pretty (P.SeqRP _ rs) =
+            myFsep $ text "(|" : (punctuate comma . map pretty $ rs) ++ [text "|)"]
         pretty (P.GuardRP _ r gs) =
-                myFsep $ text "(|" : pretty r : char '|' : map pretty gs ++ [text "|)"]
+                myFsep $ text "(|" : pretty r : char '|' :
+                           (punctuate comma . map pretty $ gs) ++ [text "|)"]
         pretty (P.EitherRP _ r1 r2) = parens . myFsep $ [pretty r1, char '|', pretty r2]
         pretty (P.CAsRP _ n (P.IrrPat _ e)) =
                 myFsep [pretty n <> text "@:", char '~' <> pretty e]
@@ -1797,12 +1912,13 @@ instance SrcInfo loc => Pretty (P.PContext loc) where
         pretty (P.CxTuple _ assts) = myFsep [parenList (map pretty assts), text "=>"]
 
 instance SrcInfo loc => Pretty (P.PAsst loc) where
-        pretty (P.ClassA _ a ts)   = myFsep $ ppQName (sQName a) : map (prettyPrec prec_atype) ts
-        pretty (P.VarA _ n)        = pretty n
+        pretty (P.ClassA _ a ts)   = myFsep $ pretty (sQName a) : map (prettyPrec prec_atype) ts
+        pretty (P.AppA _ n ns)     = myFsep $ pretty n : map (prettyPrec prec_atype) ns
         pretty (P.InfixA _ a op b) = myFsep [pretty a, ppQNameInfix (sQName op), pretty b]
         pretty (P.IParam _ i t)    = myFsep [pretty i, text "::", pretty t]
         pretty (P.EqualP _ t1 t2)  = myFsep [pretty t1, text "~", pretty t2]
         pretty (P.ParenA _ a)      = parens (pretty a)
+        pretty (P.WildCardA _ mn)  = char '_' <> maybePP pretty mn
 
 instance SrcInfo loc => Pretty (P.PType loc) where
         prettyPrec p (P.TyForall _ mtvs ctxt htype) = parensIf (p > 0) $
@@ -1829,4 +1945,5 @@ instance SrcInfo loc => Pretty (P.PType loc) where
         prettyPrec _ (P.TyKind _ t k) = parens (myFsep [pretty t, text "::", pretty k])
         prettyPrec _ (P.TyPromoted _ p) = pretty $ sPromoted p
         prettyPrec _ (P.TySplice _ s) = pretty s
-        prettyPrec _ (P.TyBang _ b t) = pretty b <> pretty t
+        prettyPrec _ (P.TyBang _ b t) = pretty b <> prettyPrec prec_atype t
+        prettyPrec _ (P.TyWildCard _ mn) = char '_' <> maybePP pretty mn
