@@ -67,7 +67,7 @@ mkSrcSpan (SrcLoc fn sl sc) (SrcLoc _ el ec) = SrcSpan fn sl sc el ec
 --   to the end of the second. Assumes that the two spans relate to the
 --   same source file.
 mergeSrcSpan :: SrcSpan -> SrcSpan -> SrcSpan
-mergeSrcSpan (SrcSpan fn sl1 sc1 el1 ec1) (SrcSpan _ sl2 sc2 el2 ec2) = 
+mergeSrcSpan (SrcSpan fn sl1 sc1 el1 ec1) (SrcSpan _ sl2 sc2 el2 ec2) =
     let (sl,sc) = min (sl1,sc1) (sl2,sc2)
         (el,ec) = max (el1,ec1) (el2,ec2)
      in SrcSpan fn sl sc el ec
@@ -96,7 +96,12 @@ data SrcSpanInfo = SrcSpanInfo
 --    , explLayout     :: Bool
     , srcInfoPoints  :: [SrcSpan]    -- Marks the location of specific entities inside the span
     }
-  deriving (Eq,Ord,Show,Typeable,Data)
+  deriving (Eq,Ord,Typeable,Data)
+
+-- Identical output to the derived show instance for GHC 7.10 and earlier.
+instance Show SrcSpanInfo where
+  showsPrec n (SrcSpanInfo s pts) = showParen (n >= 11) . showString $
+    "SrcSpanInfo {srcInfoSpan = " ++ show s ++ ", srcInfoPoints = " ++ show pts ++ "}"
 
 -- | Generate a 'SrcSpanInfo' with no positional information for entities.
 noInfoSpan :: SrcSpan -> SrcSpanInfo
@@ -113,6 +118,13 @@ combSpanInfo s1 s2 = SrcSpanInfo
     (mergeSrcSpan (srcInfoSpan s1) (srcInfoSpan s2))
     []
 
+-- | Like '(<+?>)', but it also concatenates the 'srcInfoPoints'.
+combSpanMaybe :: SrcSpanInfo -> Maybe SrcSpanInfo -> SrcSpanInfo
+combSpanMaybe s1 Nothing = s1
+combSpanMaybe s1 (Just s2) = SrcSpanInfo
+    (mergeSrcSpan (srcInfoSpan s1) (srcInfoSpan s2))
+    (srcInfoPoints s1 ++ srcInfoPoints s2)
+
 -- | Short name for 'combSpanInfo'
 (<++>) :: SrcSpanInfo -> SrcSpanInfo -> SrcSpanInfo
 (<++>) = combSpanInfo
