@@ -42,7 +42,7 @@ module Language.Haskell.Exts.Syntax (
     Module(..), WarningText(..), ExportSpec(..),
     ImportDecl(..), ImportSpec(..), Assoc(..), Namespace(..),
     -- * Declarations
-    Decl(..), Binds(..), IPBind(..),
+    Decl(..), Binds(..), IPBind(..), PatternSynDirection(..),
     -- ** Type classes and instances
     ClassDecl(..), InstDecl(..), Deriving,
     -- ** Data type declarations
@@ -63,7 +63,7 @@ module Language.Haskell.Exts.Syntax (
     Literal(..), Sign(..),
     -- * Variables, Constructors and Operators
     ModuleName(..), QName(..), Name(..), QOp(..), Op(..),
-    SpecialCon(..), CName(..), IPName(..), XName(..),
+    SpecialCon(..), CName(..), IPName(..), XName(..), Role(..),
 
     -- * Template Haskell
     Bracket(..), Splice(..),
@@ -89,7 +89,7 @@ module Language.Haskell.Exts.Syntax (
     as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name,
     export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name,
     stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name,
-    capi_name, forall_name, family_name,
+    javascript_name, capi_name, forall_name, family_name,
     -- ** Type constructors
     unit_tycon_name, fun_tycon_name, list_tycon_name, tuple_tycon_name, unboxed_singleton_tycon_name,
     unit_tycon, fun_tycon, list_tycon, tuple_tycon, unboxed_singleton_tycon,
@@ -168,8 +168,8 @@ data Module = Module SrcLoc ModuleName [ModulePragma] (Maybe WarningText)
 
 -- | An item in a module's export specification.
 data ExportSpec
-     = EVar Namespace QName         -- ^ variable.
-     | EAbs QName                   -- ^ @T@:
+     = EVar QName                   -- ^ variable.
+     | EAbs Namespace QName         -- ^ @T@:
                                     --   a class or datatype exported abstractly,
                                     --   or a type synonym.
      | EThingAll QName              -- ^ @T(..)@:
@@ -183,7 +183,7 @@ data ExportSpec
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Namespaces for imports/exports.
-data Namespace = NoNamespace | TypeNamespace
+data Namespace = NoNamespace | TypeNamespace | PatternNamespace
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An import declaration.
@@ -205,8 +205,8 @@ data ImportDecl = ImportDecl
 -- | An import specification, representing a single explicit item imported
 --   (or hidden) from a module.
 data ImportSpec
-     = IVar Namespace Name      -- ^ variable.
-     | IAbs Name                -- ^ @T@:
+     = IVar Name                -- ^ variable.
+     | IAbs Namespace Name      -- ^ @T@:
                                 --   the name of a class, datatype or type synonym.
      | IThingAll Name           -- ^ @T(..)@:
                                 --   a class imported with all of its methods, or
@@ -260,14 +260,17 @@ data Decl
      -- ^ A Template Haskell splicing declaration
      | TypeSig      SrcLoc [Name] Type
      -- ^ A type signature declaration
+     | PatSynSig    SrcLoc Name (Maybe [TyVarBind]) Context Context Type
+     -- ^ Pattern Synonym Signature
      | FunBind      [Match]
      -- ^ A set of function binding clauses
-     | PatBind      SrcLoc Pat Rhs {-where-} Binds
+     | PatBind      SrcLoc Pat Rhs {-where-} (Maybe Binds)
      -- ^ A pattern binding
      | ForImp   SrcLoc CallConv Safety String Name Type
      -- ^ A foreign import declaration
      | ForExp   SrcLoc CallConv          String Name Type
      -- ^ A foreign export declaration
+     | PatSyn   SrcLoc Pat Pat PatternSynDirection
 
      | RulePragmaDecl   SrcLoc [Rule]
      -- ^ A RULES pragma
@@ -289,6 +292,15 @@ data Decl
      -- ^ An ANN pragma
      | MinimalPragma    SrcLoc (Maybe BooleanFormula)
      -- ^ A MINIMAL pragma
+     | RoleAnnotDecl    SrcLoc QName [Role]
+     -- ^ A role annotation
+
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
+data  PatternSynDirection =
+      Unidirectional -- ^ A unidirectional pattern synonym with "<-"
+    | ImplicitBidirectional  -- ^ A bidirectional pattern synonym with "="
+    | ExplicitBidirectional [Decl]  -- ^ A birectional pattern synonym with the construction specified.
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A type equation of the form @rhs = lhs@ used in closed type families.
@@ -312,6 +324,13 @@ data BooleanFormula
     | ParenFormula BooleanFormula    -- ^ Parenthesized boolean formulas.
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
+data Role
+  = Nominal
+  | Representational
+  | Phantom
+  | RoleWildcard
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
 -- | A flag stating whether a declaration is a data or newtype declaration.
 data DataOrNew = DataType | NewType
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
@@ -328,7 +347,7 @@ data IPBind = IPBind SrcLoc IPName Exp
 
 -- | Clauses of a function binding.
 data Match
-     = Match SrcLoc Name [Pat] (Maybe Type) Rhs {-where-} Binds
+     = Match SrcLoc Name [Pat] (Maybe Type) Rhs {-where-} (Maybe Binds)
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A single constructor declaration within a data type declaration,
@@ -424,14 +443,15 @@ data Type
      | TyEquals Type Type       -- ^ type equality predicate enabled by ConstraintKinds
      | TySplice Splice          -- ^ template haskell splice type
      | TyBang BangType Type     -- ^ Strict type marked with \"@!@\" or type marked with UNPACK pragma.
+     | TyWildCard (Maybe Name)  -- ^ Type wildcard
   deriving (Eq,Ord,Show,Typeable,Data)
 
 data Promoted
         = PromotedInteger Integer
         | PromotedString String
         | PromotedCon Bool QName
-        | PromotedList Bool [Promoted]
-        | PromotedTuple [Promoted]
+        | PromotedList Bool [Type]
+        | PromotedTuple [Type]
         | PromotedUnit
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
@@ -444,13 +464,12 @@ data TyVarBind
 -- | An explicit kind annotation.
 data Kind
     = KindStar          -- ^ @*@, the kind of types
-    | KindBang          -- ^ @!@, the kind of unboxed types
     | KindFn Kind Kind  -- ^ @->@, the kind of a type constructor
     | KindParen Kind    -- ^ a kind surrounded by parentheses
     | KindVar QName     -- ^ a kind variable (as of yet unsupported by compilers)
     | KindApp Kind Kind -- ^ @k1 k2@
-    | KindTuple [Kind]  -- ^ @'(k1,k2,k3)@, a promoted tuple
-    | KindList  [Kind]  -- ^ @'[k1,k2,k3]@, a promoted list literal
+    | KindTuple [Kind]  -- ^ @(k1,k2,k3)@, kind of a promoted tuple
+    | KindList Kind     -- ^ @[k1]@, kind of a promoted list
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 
@@ -468,11 +487,12 @@ type Context = [Asst]
 --   allows multiple parameters, and allows them to be /type/s.
 --   Also extended with support for implicit parameters and equality constraints.
 data Asst = ClassA QName [Type]     -- ^ ordinary class assertion
-          | VarA Name               -- ^ constraint kind assertion, @Dict :: cxt => Dict cxt@
+          | AppA Name [Type]        -- ^ constraint kind assertion, @Dict :: cxt a => Dict cxt@
           | InfixA Type QName Type  -- ^ class assertion where the class name is given infix
           | IParam IPName Type      -- ^ implicit parameter assertion
           | EqualP Type   Type      -- ^ type equality constraint
           | ParenA Asst             -- ^ parenthesised class assertion
+          | WildCardA (Maybe Name) -- ^ A wildcard
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | /literal/
@@ -577,6 +597,9 @@ data Exp
 
 -- LambdaCase
     | LCase [Alt] -- ^ @\case@ /alts/
+
+-- Holes
+    | ExprHole                  -- ^ Expression hole
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | The name of an xml element or attribute,
@@ -619,6 +642,7 @@ data CallConv
     | DotNet
     | Jvm
     | Js
+    | JavaScript
     | CApi
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
@@ -760,7 +784,7 @@ data FieldUpdate
 
 -- | An /alt/ alternative in a @case@ expression.
 data Alt
-    = Alt SrcLoc Pat Rhs Binds
+    = Alt SrcLoc Pat Rhs (Maybe Binds)
   deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -----------------------------------------------------------------------------
@@ -805,7 +829,8 @@ star_name      = Symbol "*"
 
 export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name,
   stdcall_name, ccall_name, cplusplus_name, dotnet_name,
-  jvm_name, js_name, capi_name, forall_name, family_name :: Name
+  jvm_name, js_name, javascript_name, capi_name, forall_name,
+  family_name :: Name
 export_name     = Ident "export"
 safe_name       = Ident "safe"
 unsafe_name     = Ident "unsafe"
@@ -817,6 +842,7 @@ cplusplus_name  = Ident "cplusplus"
 dotnet_name     = Ident "dotnet"
 jvm_name        = Ident "jvm"
 js_name         = Ident "js"
+javascript_name = Ident "js"
 capi_name       = Ident "capi"
 forall_name     = Ident "forall"
 family_name     = Ident "family"
